# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from pontos.testing import temp_directory, temp_python_module
from pontos.version.project import Project, ProjectError
from pontos.version.schemes import PEP440VersioningScheme


class ProjectTestCase(unittest.TestCase):
    def test_no_project_found(self):
        with (
            temp_directory(change_into=True),
            self.assertRaisesRegex(
                ProjectError, "No project settings file found"
            ),
        ):
            Project(PEP440VersioningScheme)

    def test_python_project(self):
        current_version = PEP440VersioningScheme.parse_version("1.2.3")
        new_version = PEP440VersioningScheme.parse_version("1.2.4")

        content = f"__version__ = '{current_version}'"
        with temp_python_module(
            content, name="foo", change_into=True
        ) as temp_module:
            pyproject_toml = temp_module.parent / "pyproject.toml"
            pyproject_toml.write_text(
                f'[tool.poetry]\nversion = "{current_version}"\n'
                '[tool.pontos.version]\nversion-module-file = "foo.py"',
                encoding="utf8",
            )

            project = Project(PEP440VersioningScheme)
            self.assertEqual(project.get_current_version(), current_version)

            update = project.update_version(new_version)

            self.assertEqual(update.previous, current_version)
            self.assertEqual(update.new, new_version)

            self.assertEqual(len(update.changed_files), 2)

    def test_go_project(self):
        current_version = PEP440VersioningScheme.parse_version("1.2.3")
        new_version = PEP440VersioningScheme.parse_version("1.2.4")

        with temp_directory(change_into=True) as temp_dir:
            project_file = temp_dir / "go.mod"
            project_file.touch()
            version_file = temp_dir / "version.go"
            version_file.write_text(f'var version = "{current_version}"')

            project = Project(PEP440VersioningScheme)
            self.assertEqual(project.get_current_version(), current_version)

            update = project.update_version(new_version)

            self.assertEqual(update.previous, current_version)
            self.assertEqual(update.new, new_version)

            self.assertEqual(len(update.changed_files), 1)

    def test_javascript_project(self):
        current_version = PEP440VersioningScheme.parse_version("1.2.3")
        new_version = PEP440VersioningScheme.parse_version("1.2.4")

        with temp_directory(change_into=True) as temp_dir:
            version_file = temp_dir / "package.json"
            version_file.write_text(
                f'{{"name": "foo", "version": "{current_version}"}}',
                encoding="utf8",
            )

            project = Project(PEP440VersioningScheme)
            self.assertEqual(project.get_current_version(), current_version)

            update = project.update_version(new_version)

            self.assertEqual(update.previous, current_version)
            self.assertEqual(update.new, new_version)

            self.assertEqual(len(update.changed_files), 1)

    def test_cmake_project_version(self):
        current_version = PEP440VersioningScheme.parse_version("1.2.3")
        new_version = PEP440VersioningScheme.parse_version("1.2.4")

        with temp_directory(change_into=True) as temp_dir:
            version_file = temp_dir / "CMakeLists.txt"
            version_file.write_text("project(VERSION 1.2.3)", encoding="utf8")

            project = Project(PEP440VersioningScheme)
            self.assertEqual(project.get_current_version(), current_version)

            update = project.update_version(new_version)

            self.assertEqual(update.previous, current_version)
            self.assertEqual(update.new, new_version)

            self.assertEqual(len(update.changed_files), 1)

    def test_all(self):
        current_version = PEP440VersioningScheme.parse_version("1.2.3")
        new_version = PEP440VersioningScheme.parse_version("1.2.4")

        content = f"__version__ = '{current_version}'"
        with temp_python_module(
            content, name="foo", change_into=True
        ) as temp_module:
            temp_dir = temp_module.parent
            pyproject_toml = temp_dir / "pyproject.toml"
            pyproject_toml.write_text(
                f'[tool.poetry]\nversion = "{current_version}"\n'
                '[tool.pontos.version]\nversion-module-file = "foo.py"',
                encoding="utf8",
            )

            go_mod_file = temp_dir / "go.mod"
            go_mod_file.touch()
            go_version_file = temp_dir / "version.go"
            go_version_file.write_text(f'var version = "{current_version}"')

            package_json = temp_dir / "package.json"
            package_json.write_text(
                f'{{"name": "foo", "version": "{current_version}"}}',
                encoding="utf8",
            )

            cmake_lists_txt = temp_dir / "CMakeLists.txt"
            cmake_lists_txt.write_text(
                "project(VERSION 1.2.3)", encoding="utf8"
            )

            project = Project(PEP440VersioningScheme)
            self.assertEqual(project.get_current_version(), current_version)

            update = project.update_version(new_version)

            self.assertEqual(update.previous, current_version)
            self.assertEqual(update.new, new_version)

            self.assertEqual(len(update.changed_files), 5)



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# pylint: disable=line-too-long
# ruff: noqa: E501

import unittest
from datetime import datetime, timezone
from uuid import UUID

from pontos.nvd.models.cve_change import CVEChange, Detail, EventName
from tests.nvd import get_cve_change_data


class CVEChangeTestCase(unittest.TestCase):
    def test_required_only(self):
        cve_change = CVEChange.from_dict(get_cve_change_data())

        self.assertEqual(cve_change.cve_id, "CVE-2022-0001")
        self.assertEqual(cve_change.event_name, "Initial Analysis")
        self.assertEqual(
            cve_change.cve_change_id,
            UUID("5160FDEB-0FF0-457B-AA36-0AEDCAB2522E"),
        )
        self.assertEqual(cve_change.source_identifier, "nvd@nist.gov")
        self.assertEqual(
            cve_change.created,
            datetime(2022, 3, 18, 20, 13, 8, 123000, tzinfo=timezone.utc),
        )
        self.assertEqual(
            cve_change.details,
            [
                Detail(
                    action="Added",
                    type="CVSS V2",
                    new_value="NIST (AV:L/AC:L/Au:N/C:P/I:N/A:N)",
                ),
                Detail(
                    action="Added",
                    type="CVSS V3.1",
                    new_value="NIST AV:L/AC:L/PR:L/UI:N/S:C/C:H/I:N/A:N",
                ),
                Detail(
                    action="Changed",
                    type="Reference Type",
                    old_value="http://www.openwall.com/lists/oss-security/2022/03/18/2 No Types Assigned",
                    new_value="http://www.openwall.com/lists/oss-security/2022/03/18/2 Mailing List, Third Party Advisory",
                ),
            ],
        )


class EventNameTestCase(unittest.TestCase):
    def test_init(self):
        self.assertEqual(
            EventName("Initial Analysis"), EventName.INITIAL_ANALYSIS
        )

    def test__str__(self):
        self.assertEqual(str(EventName.INITIAL_ANALYSIS), "Initial Analysis")



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from datetime import datetime, timezone
from typing import Any, Optional
from unittest.mock import MagicMock, patch
from uuid import UUID

from httpx import AsyncClient, Response

from pontos.errors import PontosError
from pontos.nvd.api import now
from pontos.nvd.cve_changes.api import MAX_CVE_CHANGES_PER_PAGE, CVEChangesApi
from pontos.nvd.models.cve_change import Detail, EventName
from tests import AsyncMock, IsolatedAsyncioTestCase, aiter, anext
from tests.nvd import get_cve_change_data


def create_cve_changes_response(
    cve_id: str, update: Optional[dict[str, Any]] = None
) -> MagicMock:
    data = {
        "cve_changes": [{"change": get_cve_change_data({"cve_id": cve_id})}],
        "results_per_page": 1,
    }
    if update:
        data.update(update)

    response = MagicMock(spec=Response)
    response.json.return_value = data
    return response


def create_cve_changes_responses(count: int = 2) -> list[MagicMock]:
    return [
        create_cve_changes_response(f"CVE-{i}", {"total_results": count})
        for i in range(1, count + 1)
    ]


class CVEChangesApiTestCase(IsolatedAsyncioTestCase):
    @patch("pontos.nvd.api.AsyncClient", spec=AsyncClient)
    def setUp(self, async_client: MagicMock) -> None:
        self.http_client = AsyncMock()
        async_client.return_value = self.http_client
        self.api = CVEChangesApi(token="token")

    async def test_cve_changes(self):
        self.http_client.get.side_effect = create_cve_changes_responses()

        it = aiter(self.api.changes())
        cve_change = await anext(it)

        self.assertEqual(cve_change.cve_id, "CVE-1")
        self.assertEqual(cve_change.event_name, EventName.INITIAL_ANALYSIS)
        self.assertEqual(
            cve_change.cve_change_id,
            UUID("5160FDEB-0FF0-457B-AA36-0AEDCAB2522E"),
        )
        self.assertEqual(cve_change.source_identifier, "nvd@nist.gov")
        self.assertEqual(
            cve_change.created,
            datetime(2022, 3, 18, 20, 13, 8, 123000, tzinfo=timezone.utc),
        )
        self.assertEqual(
            cve_change.details,
            [
                Detail(
                    action="Added",
                    type="CVSS V2",
                    new_value="NIST (AV:L/AC:L/Au:N/C:P/I:N/A:N)",
                ),
                Detail(
                    action="Added",
                    type="CVSS V3.1",
                    new_value="NIST AV:L/AC:L/PR:L/UI:N/S:C/C:H/I:N/A:N",
                ),
                Detail(
                    action="Changed",
                    type="Reference Type",
                    old_value="http://www.openwall.com/lists/oss-security/2022/03/18/2 No Types Assigned",
                    new_value="http://www.openwall.com/lists/oss-security/2022/03/18/2 Mailing List, Third Party Advisory",
                ),
            ],
        )

        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cvehistory/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "resultsPerPage": MAX_CVE_CHANGES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve_change = await anext(it)
        self.assertEqual(cve_change.cve_id, "CVE-2")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cvehistory/2.0",
            headers={"apiKey": "token"},
            params={"startIndex": 1, "resultsPerPage": 1},
        )

        with self.assertRaises(StopAsyncIteration):
            cve_change = await anext(it)

    async def test_cve_changes_change_dates(self):
        self.http_client.get.side_effect = create_cve_changes_responses()

        it = aiter(
            self.api.changes(
                change_start_date=datetime(2022, 12, 1),
                change_end_date=datetime(2022, 12, 31),
            )
        )
        cve_changes = await anext(it)

        self.assertEqual(cve_changes.cve_id, "CVE-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cvehistory/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "changeStartDate": "2022-12-01T00:00:00",
                "changeEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": MAX_CVE_CHANGES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve_changes = await anext(it)

        self.assertEqual(cve_changes.cve_id, "CVE-2")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cvehistory/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "changeStartDate": "2022-12-01T00:00:00",
                "changeEndDate": "2022-12-31T00:00:00",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve_changes = await anext(it)

    async def test_cve_changes_cve_id(self):
        self.http_client.get.side_effect = create_cve_changes_responses()

        it = aiter(self.api.changes(cve_id="CVE-1"))
        cve_changes = await anext(it)

        self.assertEqual(cve_changes.cve_id, "CVE-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cvehistory/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "cveId": "CVE-1",
                "resultsPerPage": MAX_CVE_CHANGES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve_changes = await anext(it)

        self.assertEqual(cve_changes.cve_id, "CVE-2")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cvehistory/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "cveId": "CVE-1",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            cve_changes = await anext(it)

    async def test_cve_changes_event_name(self):
        self.http_client.get.side_effect = create_cve_changes_responses()

        it = aiter(self.api.changes(event_name=EventName.INITIAL_ANALYSIS))
        cve_changes = await anext(it)

        self.assertEqual(cve_changes.cve_id, "CVE-1")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cvehistory/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "eventName": "Initial Analysis",
                "resultsPerPage": MAX_CVE_CHANGES_PER_PAGE,
            },
        )

        self.http_client.get.reset_mock()

        cve_changes = await anext(it)

        self.assertEqual(cve_changes.cve_id, "CVE-2")
        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cvehistory/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 1,
                "eventName": "Initial Analysis",
                "resultsPerPage": 1,
            },
        )

        with self.assertRaises(StopAsyncIteration):
            await anext(it)

    @patch("pontos.nvd.cve_changes.api.now", spec=now)
    async def test_cve_changes_calculate_end_date(self, now_mock: MagicMock):
        now_mock.return_value = datetime(2023, 1, 2, tzinfo=timezone.utc)
        self.http_client.get.side_effect = create_cve_changes_responses()

        it = aiter(
            self.api.changes(
                change_start_date=datetime(2023, 1, 1, tzinfo=timezone.utc)
            )
        )

        await anext(it)

        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cvehistory/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "changeStartDate": "2023-01-01T00:00:00+00:00",
                "changeEndDate": "2023-01-02T00:00:00+00:00",
                "resultsPerPage": MAX_CVE_CHANGES_PER_PAGE,
            },
        )

    @patch("pontos.nvd.cve_changes.api.now", spec=now)
    async def test_cve_changes_calculate_end_date_with_limit(
        self, now_mock: MagicMock
    ):
        now_mock.return_value = datetime(2023, 5, 2, tzinfo=timezone.utc)
        self.http_client.get.side_effect = create_cve_changes_responses()

        it = aiter(
            self.api.changes(
                change_start_date=datetime(2023, 1, 1, tzinfo=timezone.utc)
            )
        )

        await anext(it)

        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cvehistory/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "changeStartDate": "2023-01-01T00:00:00+00:00",
                "changeEndDate": "2023-05-01T00:00:00+00:00",
                "resultsPerPage": MAX_CVE_CHANGES_PER_PAGE,
            },
        )

    async def test_cve_changes_calculate_start_date(self):
        self.http_client.get.side_effect = create_cve_changes_responses()

        it = aiter(
            self.api.changes(
                change_end_date=datetime(2023, 5, 1, tzinfo=timezone.utc)
            )
        )

        await anext(it)

        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cvehistory/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "changeStartDate": "2023-01-01T00:00:00+00:00",
                "changeEndDate": "2023-05-01T00:00:00+00:00",
                "resultsPerPage": MAX_CVE_CHANGES_PER_PAGE,
            },
        )

    async def test_cve_changes_range_too_long(self):
        with self.assertRaises(PontosError):
            self.api.changes(
                change_start_date=datetime(2023, 1, 1),
                change_end_date=datetime(2023, 5, 2),
            )

    async def test_cve_changes_request_results(self):
        self.http_client.get.side_effect = create_cve_changes_responses()

        it = aiter(self.api.changes(request_results=10))

        await anext(it)

        self.http_client.get.assert_awaited_once_with(
            "https://services.nvd.nist.gov/rest/json/cvehistory/2.0",
            headers={"apiKey": "token"},
            params={
                "startIndex": 0,
                "resultsPerPage": 10,
            },
        )

    async def test_context_manager(self):
        async with self.api:
            pass

        self.http_client.__aenter__.assert_awaited_once()
        self.http_client.__aexit__.assert_awaited_once()



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import AsyncIterator, Union

from pontos.github.api.client import GitHubAsyncREST
from pontos.github.api.helper import JSON_OBJECT
from pontos.github.models.base import User
from pontos.github.models.user import (
    EmailInformation,
    SSHPublicKey,
    SSHPublicKeyExtended,
)


class GitHubAsyncRESTUsers(GitHubAsyncREST):
    async def users(self) -> AsyncIterator[User]:
        """

        https://docs.github.com/en/rest/users/users#list-users

        Args:
            username: The handle for the GitHub user account

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding user information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for user in api.users.users():
                        print(user)
        """
        api = "/users"
        params = {"per_page": "100"}
        async for response in self._client.get_all(api, params=params):
            response.raise_for_status()

            for user in response.json():
                yield User.from_dict(user)

    async def user(self, username: str) -> User:
        """
        Provide publicly available information about someone with a GitHub
        account

        https://docs.github.com/en/rest/users/users#get-a-user

        Args:
            username: The handle for the GitHub user account

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Information about the user

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    user = await api.users.user("foo")
                    print(user)
        """
        api = f"/users/{username}"
        response = await self._client.get(api)
        response.raise_for_status()
        return User.from_dict(response.json())

    async def current_user(self) -> User:
        """
        Get the current authenticated user

        https://docs.github.com/en/rest/users/users#get-the-authenticated-user

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Information about the user

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    user = await api.users.current_user()
                    print(user)
        """
        api = "/user"
        response = await self._client.get(api)
        response.raise_for_status()
        return User.from_dict(response.json())

    async def user_keys(self, username: str) -> AsyncIterator[SSHPublicKey]:
        """
        List the verified public SSH keys for a user

        https://docs.github.com/en/rest/users/keys#list-public-keys-for-a-user

        Args:
            username: The handle for the GitHub user account

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding ssh key information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for key in api.users.user_keys("foo"):
                        print(key)
        """
        api = f"/users/{username}/keys"
        params = {"per_page": "100"}
        async for response in self._client.get_all(api, params=params):
            response.raise_for_status()

            for key in response.json():
                yield SSHPublicKey.from_dict(key)

    async def keys(self) -> AsyncIterator[SSHPublicKey]:
        """
        List the public SSH keys for the authenticated user's GitHub account

        https://docs.github.com/en/rest/users/keys#list-public-ssh-keys-for-the-authenticated-user

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding ssh key information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for key in api.users.keys():
                        print(key)
        """
        api = "/user/keys"
        params = {"per_page": "100"}
        async for response in self._client.get_all(api, params=params):
            response.raise_for_status()

            for key in response.json():
                yield SSHPublicKey.from_dict(key)

    async def emails(self) -> AsyncIterator[EmailInformation]:
        """
        List all email addresses of the currently logged in user

        https://docs.github.com/en/rest/users/emails#list-email-addresses-for-the-authenticated-user

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding email information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for email in api.users.emails():
                        print(email)
        """
        api = "/user/emails"
        params = {"per_page": "100"}
        async for response in self._client.get_all(api, params=params):
            response.raise_for_status()

            for email in response.json():
                yield EmailInformation.from_dict(email)

    async def key(self, key_id: Union[str, int]) -> SSHPublicKeyExtended:
        """
        View extended details for a single public SSH key

        https://docs.github.com/en/rest/users/keys#get-a-public-ssh-key-for-the-authenticated-user

        Args:
            key_id: The unique identifier of the key

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Extended information about the SSH key

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    key = await api.users.key(123)
                    print(key)
        """
        api = f"/user/keys/{key_id}"
        response = await self._client.get(api)
        response.raise_for_status()
        return SSHPublicKeyExtended.from_dict(response.json())

    async def delete_key(self, key_id: Union[str, int]) -> None:
        """
        Removes a public SSH key from the authenticated user's GitHub account

        https://docs.github.com/en/rest/users/keys#delete-a-public-ssh-key-for-the-authenticated-user

        Args:
            key_id: The unique identifier of the key

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    await api.users.delete_key(123)
        """
        api = f"/user/keys/{key_id}"
        response = await self._client.delete(api)
        response.raise_for_status()

    async def create_key(self, title: str, key: str) -> SSHPublicKeyExtended:
        """
        Adds a public SSH key to the authenticated user's GitHub account

        https://docs.github.com/en/rest/users/keys#create-a-public-ssh-key-for-the-authenticated-user

        Args:
            title: A descriptive name for the new key
            key: The public SSH key to add to your GitHub account

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Extended information about the SSH key

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    key = await api.users.create_key(
                        "My SSH Public Key",
                        "2Sg8iYjAxxmI2LvUXpJjkYrMxURPc8r+dB7TJyvv1234"
                    )
                    print(key)
        """
        api = "/user/keys"
        data: JSON_OBJECT = {"key": key, "title": title}
        response = await self._client.post(api, data=data)
        response.raise_for_status()
        return SSHPublicKeyExtended.from_dict(response.json())



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from typing import AsyncIterator, Optional, Union

from pontos.github.api.client import GitHubAsyncREST
from pontos.github.models.base import SortOrder
from pontos.github.models.dependabot import (
    AlertSort,
    AlertState,
    DependabotAlert,
    DependencyScope,
    DismissedReason,
    Severity,
)
from pontos.helper import enum_or_value


class GitHubAsyncRESTDependabot(GitHubAsyncREST):
    async def _alerts(
        self,
        api: str,
        *,
        state: Union[AlertState, str, None] = None,
        severity: Union[Severity, str, None] = None,
        ecosystem: Optional[str] = None,
        packages: Optional[list[str]] = None,
        scope: Union[DependencyScope, str, None] = None,
        sort: Union[AlertSort, str] = AlertSort.CREATED,
        direction: Union[str, SortOrder] = SortOrder.DESC,
    ) -> AsyncIterator[DependabotAlert]:
        params = {"per_page": "100"}

        if state:
            params["state"] = enum_or_value(state)
        if severity:
            params["severity"] = enum_or_value(severity)
        if ecosystem:
            params["ecosystem"] = ecosystem
        if packages:
            # as per REST api docu this param is passed as package (singular!)
            params["package"] = ",".join(packages)
        if scope:
            params["scope"] = enum_or_value(scope)
        if sort:
            params["sort"] = enum_or_value(sort)
        if direction:
            params["direction"] = enum_or_value(direction)

        async for response in self._client.get_all(api, params=params):
            response.raise_for_status()

            for alert in response.json():
                yield DependabotAlert.from_dict(alert)

    async def enterprise_alerts(
        self,
        enterprise: str,
        *,
        state: Union[AlertState, str, None] = None,
        severity: Union[Severity, str, None] = None,
        ecosystem: Optional[str] = None,
        packages: Optional[list[str]] = None,
        scope: Union[DependencyScope, str, None] = None,
        sort: Union[AlertSort, str] = AlertSort.CREATED,
        direction: Union[str, SortOrder] = SortOrder.DESC,
    ) -> AsyncIterator[DependabotAlert]:
        """
        Get the list of dependabot alerts for all repositories of a GitHub
        enterprise

        https://docs.github.com/en/rest/dependabot/alerts#list-dependabot-alerts-for-an-enterprise

        Args:
            enterprise: Name of the enterprise
            state: Filter alerts by state
            severity: Filter alerts by severity
            ecosystem: Filter alerts by package ecosystem
            package: Return alerts only for the provided packages
            scope: Filter alerts by scope of the vulnerable dependency
            sort: The property by which to sort the results. Default is to sort
                alerts by creation date.
            direction: The direction to sort the results by. Default is desc.

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding the dependabot alerts

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for alert in api.dependabot.enterprise_alerts(
                        "my-enterprise"
                    ):
                        print(alert)
        """

        api = f"/enterprises/{enterprise}/dependabot/alerts"
        async for alert in self._alerts(
            api,
            state=state,
            severity=severity,
            ecosystem=ecosystem,
            packages=packages,
            scope=scope,
            sort=sort,
            direction=direction,
        ):
            yield alert

    async def organization_alerts(
        self,
        organization: str,
        *,
        state: Union[AlertState, str, None] = None,
        severity: Union[Severity, str, None] = None,
        ecosystem: Optional[str] = None,
        packages: Optional[list[str]] = None,
        scope: Union[DependencyScope, str, None] = None,
        sort: Union[AlertSort, str] = AlertSort.CREATED,
        direction: Union[str, SortOrder] = SortOrder.DESC,
    ) -> AsyncIterator[DependabotAlert]:
        """
        Get the list of dependabot alerts for all repositories of a GitHub
        organization

        https://docs.github.com/en/rest/dependabot/alerts#list-dependabot-alerts-for-an-organization

        Args:
            organization: Name of the organization
            state: Filter alerts by state
            severity: Filter alerts by severity
            ecosystem: Filter alerts by package ecosystem
            package: Return alerts only for the provided packages
            scope: Filter alerts by scope of the vulnerable dependency
            sort: The property by which to sort the results. Default is to sort
                alerts by creation date.
            direction: The direction to sort the results by. Default is desc.

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding the dependabot alerts

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for alert in api.dependabot.organization_alerts(
                        "my-enterprise"
                    ):
                        print(alert)
        """
        api = f"/orgs/{organization}/dependabot/alerts"

        async for alert in self._alerts(
            api,
            state=state,
            severity=severity,
            ecosystem=ecosystem,
            packages=packages,
            scope=scope,
            sort=sort,
            direction=direction,
        ):
            yield alert

    async def alerts(
        self,
        repo: str,
        *,
        state: Union[AlertState, str, None] = None,
        severity: Union[Severity, str, None] = None,
        ecosystem: Optional[str] = None,
        packages: Optional[list[str]] = None,
        scope: Union[DependencyScope, str, None] = None,
        sort: Union[AlertSort, str] = AlertSort.CREATED,
        direction: Union[str, SortOrder] = SortOrder.DESC,
    ) -> AsyncIterator[DependabotAlert]:
        """
        Get the list of dependabot alerts for a repository

        https://docs.github.com/en/rest/dependabot/alerts#list-dependabot-alerts-for-a-repository

        Args:
            repo: GitHub repository (owner/name)
            state: Filter alerts by state
            severity: Filter alerts by severity
            ecosystem: Filter alerts by package ecosystem
            package: Return alerts only for the provided packages
            scope: Filter alerts by scope of the vulnerable dependency
            sort: The property by which to sort the results. Default is to sort
                alerts by creation date.
            direction: The direction to sort the results by. Default is desc.

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            An async iterator yielding the dependabot alerts

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    async for alert in api.dependabot.alerts(
                        "my-enterprise"
                    ):
                        print(alert)
        """
        api = f"/repos/{repo}/dependabot/alerts"

        async for alert in self._alerts(
            api,
            state=state,
            severity=severity,
            ecosystem=ecosystem,
            packages=packages,
            scope=scope,
            sort=sort,
            direction=direction,
        ):
            yield alert

    async def alert(
        self,
        repo: str,
        alert_number: Union[str, int],
    ) -> DependabotAlert:
        """
        Get a single dependabot alert

        https://docs.github.com/en/rest/dependabot/alerts#get-a-dependabot-alert

        Args:
            repo: GitHub repository (owner/name)
            alert_number: The number that identifies a Dependabot alert in its
                repository

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Dependabot alert information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    alert = await api.dependabot.alert("foo/bar", 123)
        """
        api = f"/repos/{repo}/dependabot/alerts/{alert_number}"
        response = await self._client.get(api)
        response.raise_for_status()
        return DependabotAlert.from_dict(response.json())

    async def update_alert(
        self,
        repo: str,
        alert_number: Union[str, int],
        state: Union[AlertState, str],
        *,
        dismissed_reason: Union[DismissedReason, str, None] = None,
        dismissed_comment: str,
    ) -> DependabotAlert:
        """
        Update a single dependabot alert

        https://docs.github.com/en/rest/dependabot/alerts#update-a-dependabot-alert

        Args:
            repo: GitHub repository (owner/name)
            alert_number: The number that identifies a Dependabot alert in its
                repository
            state: The state of the Dependabot alert
            dismissed_reason: Required when state is dismissed. A reason for
                dismissing the alert.
            dismissed_comment: An optional comment associated with dismissing
                the alert

        Raises:
            HTTPStatusError: A httpx.HTTPStatusError is raised if the request
                failed.

        Returns:
            Dependabot alert information

        Example:
            .. code-block:: python

                from pontos.github.api import GitHubAsyncRESTApi

                async with GitHubAsyncRESTApi(token) as api:
                    alert = await api.dependabot.update(
                        "foo/bar",
                        123,
                        AlertState.FIXED,
                    )
        """
        api = f"/repos/{repo}/dependabot/alerts/{alert_number}"

        data = {"state": enum_or_value(state)}
        if dismissed_comment:
            data["dismissed_comment"] = dismissed_comment
        if dismissed_reason:
            data["dismissed_reason"] = enum_or_value(dismissed_reason)

        response = await self._client.patch(api, data=data)
        response.raise_for_status()
        return DependabotAlert.from_dict(response.json())



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

"""
This script prints out all members existing in the space of the given
organization
"""

from argparse import ArgumentParser, Namespace
from typing import Union

from rich.console import Console
from rich.table import Table

from pontos.github.api import GitHubAsyncRESTApi
from pontos.github.api.organizations import MemberFilter, MemberRole


def member_filter_type(value: Union[str, MemberFilter]) -> MemberFilter:
    if isinstance(value, MemberFilter):
        return value
    return MemberFilter[value.upper()]


def member_role_type(value: Union[str, MemberRole]) -> MemberRole:
    if isinstance(value, MemberRole):
        return value
    return MemberRole[value.upper()]


def add_script_arguments(parser: ArgumentParser) -> None:
    parser.add_argument("organization")

    parser.add_argument(
        "-f",
        "--filter",
        type=member_filter_type,
        help=f"Filter members. Choices: "
        f"{', '.join([f.name for f in MemberFilter])}. Default: %(default)s",
        default=MemberFilter.ALL.name,
    )

    parser.add_argument(
        "-r",
        "--role",
        type=member_role_type,
        help=f"Show only members in specific role. Choices: "
        f"{', '.join([f.name for f in MemberRole])}. Default: %(default)s",
        default=MemberRole.ALL.name,
    )


async def github_script(api: GitHubAsyncRESTApi, args: Namespace) -> int:
    table = Table()
    table.add_column("Name")
    table.add_column("URL")

    member_count = 0
    async for user in api.organizations.members(
        args.organization, member_filter=args.filter, role=args.role
    ):
        table.add_row(
            user.login,
            f"[link={user.html_url}]{user.html_url}[/link]",
        )
        member_count += 1

    console = Console()
    console.print(table)

    print(f"{member_count} members.")
    return 0



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import os
from argparse import ArgumentParser

import shtab

from pontos.github.api.helper import DEFAULT_TIMEOUT

GITHUB_TOKEN = "GITHUB_TOKEN"


def create_parser() -> ArgumentParser:
    """
    Create a CLI parser for running Pontos GitHub Scripts

    Returns:
        A new ArgumentParser instance add the default arguments
    """
    parser = ArgumentParser(add_help=False)
    shtab.add_argument_to(parser)
    parser.add_argument(
        "--token",
        default=os.environ.get(GITHUB_TOKEN),
        help="GitHub Token. Defaults to GITHUB_TOKEN environment variable.",
    )
    parser.add_argument(
        "--timeout",
        default=DEFAULT_TIMEOUT,
        help="Timeout in seconds. Default: %(default)s.",
        type=float,
    )
    parser.add_argument("script", help="Script to run")
    return parser



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from pontos.github.api.packages import GitHubAsyncRESTPackages
from pontos.github.models.packages import (
    Package,
    PackageType,
    PackageVersion,
    PackageVisibility,
)
from pontos.testing import AsyncIteratorMock
from tests import aiter, anext
from tests.github.api import GitHubAsyncRESTTestCase, create_response

from .test_organizations import MEMBER_DICT, REPOSITORY_DICT

PACKAGE_VERSION = {
    "id": 1,
    "name": "v1.0.0",
    "url": "https://api.github.com/orgs/foo/packages/container/bar/versions/1",
    "package_html_url": "https://github.com/orgs/foo/packages/container/bar/versions",
    "created_at": "2022-01-01T00:00:00Z",
    "updated_at": "2022-01-01T00:00:00Z",
    "html_url": "https://github.com/orgs/foo/packages/container/bar/1",
    "metadata": {
        "package_type": "container",
        "container": {"tags": ["latest"]},
    },
}


class GitHubAsyncRESTPackagesTestCase(GitHubAsyncRESTTestCase):
    api_cls = GitHubAsyncRESTPackages

    async def test_exists(self):
        response = create_response(is_success=True)
        self.client.get.return_value = response

        self.assertTrue(
            await self.api.exists(
                organization="foo",
                package_type=PackageType.CONTAINER,
                package_name="bar",
            )
        )

        self.client.get.assert_awaited_once_with(
            "/orgs/foo/packages/container/bar"
        )

    async def test_package(self):
        response = create_response()
        response.json.return_value = {
            "id": 1,
            "name": "bar",
            "package_type": "container",
            "owner": MEMBER_DICT,
            "version_count": 1,
            "visibility": "public",
            "url": "https://api.github.com/orgs/foo/packages/container/bar",
            "tags": ["foo", "bar", "baz"],
            "created_at": "2022-01-01T00:00:00Z",
            "updated_at": "2022-01-01T00:00:00Z",
            "repository": REPOSITORY_DICT,
            "html_url": "https://github.com/orgs/foo/packages/container/repo/bar",
        }

        self.client.get.return_value = response

        package = await self.api.package("foo", PackageType.CONTAINER, "bar")

        self.client.get.assert_awaited_once_with(
            "/orgs/foo/packages/container/bar"
        )

        self.assertIsInstance(package, Package)
        self.assertEqual(package.owner.login, "octocat")
        self.assertEqual(package.name, "bar")
        self.assertEqual(package.version_count, 1)
        self.assertEqual(package.visibility, PackageVisibility.PUBLIC)
        self.assertEqual(
            package.url,
            "https://api.github.com/orgs/foo/packages/container/bar",
        )
        self.assertEqual(package.tags, ["foo", "bar", "baz"])
        self.assertEqual(package.created_at, "2022-01-01T00:00:00Z")
        self.assertEqual(package.updated_at, "2022-01-01T00:00:00Z")
        self.assertEqual(
            package.html_url,
            "https://github.com/orgs/foo/packages/container/repo/bar",
        )

    async def test_packages(self):
        response = create_response()
        response.json.return_value = [
            {
                "id": 1,
                "name": "bar",
                "package_type": "container",
                "owner": MEMBER_DICT,
                "version_count": 1,
                "visibility": "public",
                "url": "https://api.github.com/orgs/foo/packages/container/bar",
                "tags": ["foo", "bar", "baz"],
                "created_at": "2022-01-01T00:00:00Z",
                "updated_at": "2022-01-01T00:00:00Z",
                "repository": REPOSITORY_DICT,
                "html_url": "https://github.com/orgs/foo/packages/container/repo/bar",
            }
        ]

        self.client.get_all.return_value = AsyncIteratorMock([response])

        async_it = aiter(
            self.api.packages("foo", package_type=PackageType.CONTAINER)
        )
        package = await anext(async_it)
        self.assertEqual(package.id, 1)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/packages/container",
            params={"per_page": "100"},
        )

    async def test_package_version(self):
        response = create_response()
        response.json.return_value = PACKAGE_VERSION

        self.client.get.return_value = response

        package_version: PackageVersion = await self.api.package_version(
            "foo", PackageType.CONTAINER, "bar", 1
        )

        self.client.get.assert_awaited_once_with(
            "/orgs/foo/packages/container/bar/versions/1"
        )

        self.assertEqual(package_version.id, 1)
        self.assertEqual(package_version.name, "v1.0.0")
        self.assertEqual(
            package_version.url,
            "https://api.github.com/orgs/foo/packages/container/bar/versions/1",
        )
        self.assertEqual(
            package_version.package_html_url,
            "https://github.com/orgs/foo/packages/container/bar/versions",
        )
        self.assertEqual(package_version.created_at, "2022-01-01T00:00:00Z")
        self.assertEqual(package_version.updated_at, "2022-01-01T00:00:00Z")
        self.assertEqual(
            package_version.html_url,
            "https://github.com/orgs/foo/packages/container/bar/1",
        )
        self.assertEqual(
            package_version.metadata.package_type, PackageType.CONTAINER
        )
        self.assertEqual(package_version.metadata.container.tags, ["latest"])

    async def test_package_version_tags(self):
        response = create_response()
        response.json.return_value = {
            "metadata": {"container": {"tags": ["latest", "stable"]}}
        }

        self.client.get.return_value = response

        tags = await self.api.package_version_tags(
            organization="foo",
            package_type=PackageType.CONTAINER,
            package_name="bar",
            version=1,
        )

        self.client.get.assert_awaited_once_with(
            "/orgs/foo/packages/container/bar/versions/1"
        )

        self.assertEqual(tags, ["latest", "stable"])

    async def test_delete_package(self):
        response = create_response(is_success=True)
        self.client.delete.return_value = response

        await self.api.delete_package(
            organization="foo",
            package_type=PackageType.CONTAINER,
            package_name="bar",
        )

        self.client.delete.assert_awaited_once_with(
            "/orgs/foo/packages/container/bar"
        )

    async def test_delete_package_version(self):
        response = create_response(is_success=True)
        self.client.delete.return_value = response

        await self.api.delete_package_version(
            organization="foo",
            package_type=PackageType.CONTAINER,
            package_name="bar",
            version=1,
        )

        self.client.delete.assert_awaited_once_with(
            "/orgs/foo/packages/container/bar/versions/1"
        )

    async def test_package_versions(self):
        response1 = create_response()
        response1.json.return_value = [PACKAGE_VERSION]
        response2 = create_response()
        package_version2 = PACKAGE_VERSION.copy()
        package_version2["id"] = 2
        response2.json.return_value = [package_version2]

        self.client.get_all.return_value = AsyncIteratorMock(
            [response1, response2]
        )

        async_it = aiter(
            self.api.package_versions("foo", PackageType.CONTAINER, "bar")
        )
        package_version = await anext(async_it)
        self.assertEqual(package_version.id, 1)
        package_version = await anext(async_it)
        self.assertEqual(package_version.id, 2)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/packages/container/bar/versions"
        )

    async def test_delete_package_with_tag(self):
        response = create_response(is_success=True)
        self.client.delete.return_value = response

        package_version_response = create_response()
        package_version_response.json.return_value = [PACKAGE_VERSION]
        self.client.get_all.return_value = AsyncIteratorMock(
            [package_version_response]
        )

        tags_response = create_response()
        tags_response.json.return_value = {
            "metadata": {"container": {"tags": ["latest", "stable"]}}
        }
        self.client.get.return_value = tags_response

        await self.api.delete_package_with_tag(
            organization="foo",
            package_type=PackageType.CONTAINER,
            package_name="bar",
            tag="latest",
        )

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/packages/container/bar/versions"
        )
        self.client.get.assert_awaited_once_with(
            "/orgs/foo/packages/container/bar/versions/1"
        )
        self.client.delete.assert_awaited_once_with(
            "/orgs/foo/packages/container/bar/versions/1"
        )



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

"""
This script checks wether a package has a specific tag
"""

from argparse import ArgumentParser, Namespace

from pontos.github.api import GitHubAsyncRESTApi
from pontos.github.models.packages import PackageType


def package_type(value: str) -> PackageType:
    if isinstance(value, PackageType):
        return value
    return PackageType(value.lower())


def add_script_arguments(parser: ArgumentParser) -> None:
    parser.add_argument("organization", help="organization name")
    parser.add_argument("package", help="package name")
    parser.add_argument("tag", help="tag to check")
    parser.add_argument(
        "--package-type",
        type=package_type,
        help="package type",
        default=PackageType.CONTAINER,
    )


async def github_script(api: GitHubAsyncRESTApi, args: Namespace) -> int:
    if not await api.packages.exists(
        organization=args.organization,
        package_name=args.package,
        package_type=args.package_type,
    ):
        print(
            f"Package {args.package} does not exist in organization {args.organization}"
        )
        return 1
    print(f"Found package {args.package} in organization {args.organization}")

    async for package in api.packages.package_versions(
        organization=args.organization,
        package_name=args.package,
        package_type=args.package_type,
    ):
        print(f"Checking package {args.package} with id {package.id}")
        if package.metadata.container.tags:
            if args.tag in package.metadata.container.tags:
                print(
                    f"Package {args.package} with id {package.id} has tag {args.tag}"
                )
                tags = await api.packages.package_version_tags(
                    args.organization,
                    args.package_type,
                    args.package,
                    package.id,
                )
                print(f"Tags: {tags}")

                await api.packages.delete_package_version(
                    args.organization,
                    args.package_type,
                    args.package,
                    package.id,
                )
                return 0

    print(f"Package {args.package} does not have tag {args.tag}")
    return 0



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# pylint: disable=line-too-long
# ruff: noqa: E501

import unittest
from datetime import datetime, timezone
from uuid import UUID

from pontos.nvd.models.cpe_match_string import CPEMatch, CPEMatchString
from tests.nvd import get_cpe_match_data


class CPEMatchTestCase(unittest.TestCase):

    def test_required_only(self):
        """
        Test the required attributes of a CPEMatchString
        """
        data = get_cpe_match_data()
        data.__delitem__("matches")
        data.__delitem__("version_end_including")
        data.__delitem__("cpe_last_modified")

        cpe_match_string = CPEMatchString.from_dict(data)

        self.assertEqual(
            UUID("EAB2C9C2-F685-450B-9980-553966FC3B63"),
            cpe_match_string.match_criteria_id,
        )
        self.assertEqual(
            "cpe:2.3:a:sun:jre:*:update3:*:*:*:*:*:*",
            cpe_match_string.criteria,
        )
        self.assertEqual(
            "Active",
            cpe_match_string.status,
        )
        self.assertEqual(
            datetime(2019, 6, 17, 9, 16, 33, 960000, tzinfo=timezone.utc),
            cpe_match_string.created,
        )
        self.assertEqual(
            datetime(2019, 6, 17, 9, 16, 44, 0, tzinfo=timezone.utc),
            cpe_match_string.last_modified,
        )

        self.assertEqual([], cpe_match_string.matches)

        self.assertIsNone(cpe_match_string.cpe_last_modified)
        self.assertIsNone(cpe_match_string.version_start_excluding)
        self.assertIsNone(cpe_match_string.version_end_excluding)
        self.assertIsNone(cpe_match_string.version_start_including)
        self.assertIsNone(cpe_match_string.version_end_including)

    def test_cpe_last_modified(self):
        data = get_cpe_match_data()
        cpe_match_string = CPEMatchString.from_dict(data)

        self.assertEqual(
            datetime(2019, 7, 22, 16, 37, 38, 133000, tzinfo=timezone.utc),
            cpe_match_string.cpe_last_modified,
        )

    def test_matches(self):
        """
        Test the matches list of a CPEMatchString
        """
        cpe_match_string = CPEMatchString.from_dict(get_cpe_match_data())

        self.assertEqual(5, len(cpe_match_string.matches))

        self.assertEqual(
            CPEMatch(
                "cpe:2.3:a:sun:jre:1.3.0:update3:*:*:*:*:*:*",
                UUID("2d284534-da21-43d5-9d89-07f19ae400ea"),
            ),
            cpe_match_string.matches[0],
        )

        self.assertEqual(
            CPEMatch(
                "cpe:2.3:a:sun:jre:1.6.0:update3:*:*:*:*:*:*",
                UUID("c518a954-369e-453e-8e17-2af639150115"),
            ),
            cpe_match_string.matches[-1],
        )

    def test_including_version_limits(self):
        """
        Test the including version limits of a CPEMatchString
        """
        data = get_cpe_match_data({"version_start_including": "1.3.0"})
        cpe_match_string = CPEMatchString.from_dict(data)

        self.assertEqual("1.3.0", cpe_match_string.version_start_including)
        self.assertEqual("1.6.0", cpe_match_string.version_end_including)
        self.assertIsNone(cpe_match_string.version_start_excluding)
        self.assertIsNone(cpe_match_string.version_end_excluding)

    def test_excluding_version_limits(self):
        """
        Test the excluding version limits of a CPEMatchString
        """
        data = get_cpe_match_data(
            {
                "version_start_excluding": "1.2.0",
                "version_end_excluding": "1.7.0",
            }
        )
        data.__delitem__("version_end_including")
        cpe_match_string = CPEMatchString.from_dict(data)

        self.assertEqual("1.2.0", cpe_match_string.version_start_excluding)
        self.assertEqual("1.7.0", cpe_match_string.version_end_excluding)
        self.assertIsNone(cpe_match_string.version_start_including)
        self.assertIsNone(cpe_match_string.version_end_including)



# SPDX-FileCopyrightText: 2021-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .conventional_commits import ChangelogBuilder
from .errors import ChangelogBuilderError, ChangelogError
from .main import main

__all__ = (
    "ChangelogError",
    "ChangelogBuilderError",
    "ChangelogBuilder",
    "ConventionalCommits",
    "main",
)



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=line-too-long

import unittest
from datetime import datetime, timezone

from pontos.github.models.release import Release, ReleaseAssetState


class ReleaseTestCase(unittest.TestCase):
    def test_from_dict(self):
        release = Release.from_dict(
            {
                "url": "https://api.github.com/repos/octocat/Hello-World/releases/1",
                "html_url": "https://github.com/octocat/Hello-World/releases/v1.0.0",
                "assets_url": "https://api.github.com/repos/octocat/Hello-World/releases/1/assets",
                "upload_url": "https://uploads.github.com/repos/octocat/Hello-World/releases/1/assets{?name,label}",
                "tarball_url": "https://api.github.com/repos/octocat/Hello-World/tarball/v1.0.0",
                "zipball_url": "https://api.github.com/repos/octocat/Hello-World/zipball/v1.0.0",
                "discussion_url": "https://github.com/octocat/Hello-World/discussions/90",
                "id": 1,
                "node_id": "MDc6UmVsZWFzZTE=",
                "tag_name": "v1.0.0",
                "target_commitish": "master",
                "name": "v1.0.0",
                "body": "Description of the release",
                "draft": False,
                "prerelease": False,
                "created_at": "2013-02-27T19:35:32Z",
                "published_at": "2013-02-27T19:35:32Z",
                "author": {
                    "login": "octocat",
                    "id": 1,
                    "node_id": "MDQ6VXNlcjE=",
                    "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                    "gravatar_id": "",
                    "url": "https://api.github.com/users/octocat",
                    "html_url": "https://github.com/octocat",
                    "followers_url": "https://api.github.com/users/octocat/followers",
                    "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                    "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                    "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                    "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                    "organizations_url": "https://api.github.com/users/octocat/orgs",
                    "repos_url": "https://api.github.com/users/octocat/repos",
                    "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                    "received_events_url": "https://api.github.com/users/octocat/received_events",
                    "type": "User",
                    "site_admin": False,
                },
                "assets": [
                    {
                        "url": "https://api.github.com/repos/octocat/Hello-World/releases/assets/1",
                        "browser_download_url": "https://github.com/octocat/Hello-World/releases/download/v1.0.0/example.zip",
                        "id": 1,
                        "node_id": "MDEyOlJlbGVhc2VBc3NldDE=",
                        "name": "example.zip",
                        "label": "short description",
                        "state": "uploaded",
                        "content_type": "application/zip",
                        "size": 1024,
                        "download_count": 42,
                        "created_at": "2013-02-27T19:35:32Z",
                        "updated_at": "2013-02-27T19:35:32Z",
                        "uploader": {
                            "login": "octocat",
                            "id": 1,
                            "node_id": "MDQ6VXNlcjE=",
                            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                            "gravatar_id": "",
                            "url": "https://api.github.com/users/octocat",
                            "html_url": "https://github.com/octocat",
                            "followers_url": "https://api.github.com/users/octocat/followers",
                            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                            "organizations_url": "https://api.github.com/users/octocat/orgs",
                            "repos_url": "https://api.github.com/users/octocat/repos",
                            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                            "received_events_url": "https://api.github.com/users/octocat/received_events",
                            "type": "User",
                            "site_admin": False,
                        },
                    }
                ],
            }
        )

        self.assertEqual(
            release.url,
            "https://api.github.com/repos/octocat/Hello-World/releases/1",
        )
        self.assertEqual(
            release.html_url,
            "https://github.com/octocat/Hello-World/releases/v1.0.0",
        )
        self.assertEqual(
            release.assets_url,
            "https://api.github.com/repos/octocat/Hello-World/releases/1/assets",
        )
        self.assertEqual(
            release.upload_url,
            "https://uploads.github.com/repos/octocat/Hello-World/releases/1/assets{?name,label}",
        )
        self.assertEqual(
            release.tarball_url,
            "https://api.github.com/repos/octocat/Hello-World/tarball/v1.0.0",
        )
        self.assertEqual(
            release.zipball_url,
            "https://api.github.com/repos/octocat/Hello-World/zipball/v1.0.0",
        )
        self.assertEqual(
            release.discussion_url,
            "https://github.com/octocat/Hello-World/discussions/90",
        )
        self.assertEqual(release.id, 1)
        self.assertEqual(release.node_id, "MDc6UmVsZWFzZTE=")
        self.assertEqual(release.tag_name, "v1.0.0")
        self.assertEqual(release.target_commitish, "master")
        self.assertEqual(release.name, "v1.0.0")
        self.assertEqual(release.body, "Description of the release")
        self.assertFalse(release.draft)
        self.assertFalse(release.prerelease)
        self.assertEqual(
            release.created_at,
            datetime(2013, 2, 27, 19, 35, 32, tzinfo=timezone.utc),
        )
        self.assertEqual(
            release.published_at,
            datetime(2013, 2, 27, 19, 35, 32, tzinfo=timezone.utc),
        )

        user = release.author
        self.assertEqual(user.login, "octocat")
        self.assertEqual(user.id, 1)
        self.assertEqual(user.node_id, "MDQ6VXNlcjE=")
        self.assertEqual(
            user.avatar_url, "https://github.com/images/error/octocat_happy.gif"
        )
        self.assertEqual(user.gravatar_id, "")
        self.assertEqual(user.url, "https://api.github.com/users/octocat")
        self.assertEqual(user.html_url, "https://github.com/octocat")
        self.assertEqual(
            user.followers_url, "https://api.github.com/users/octocat/followers"
        )
        self.assertEqual(
            user.following_url,
            "https://api.github.com/users/octocat/following{/other_user}",
        )
        self.assertEqual(
            user.gists_url,
            "https://api.github.com/users/octocat/gists{/gist_id}",
        )
        self.assertEqual(
            user.starred_url,
            "https://api.github.com/users/octocat/starred{/owner}{/repo}",
        )
        self.assertEqual(
            user.subscriptions_url,
            "https://api.github.com/users/octocat/subscriptions",
        )
        self.assertEqual(
            user.organizations_url, "https://api.github.com/users/octocat/orgs"
        )
        self.assertEqual(
            user.repos_url, "https://api.github.com/users/octocat/repos"
        )
        self.assertEqual(
            user.events_url,
            "https://api.github.com/users/octocat/events{/privacy}",
        )
        self.assertEqual(
            user.received_events_url,
            "https://api.github.com/users/octocat/received_events",
        )
        self.assertEqual(user.type, "User")
        self.assertEqual(user.site_admin, False)

        self.assertEqual(len(release.assets), 1)
        asset = release.assets[0]
        self.assertEqual(
            asset.url,
            "https://api.github.com/repos/octocat/Hello-World/releases/assets/1",
        )
        self.assertEqual(
            asset.browser_download_url,
            "https://github.com/octocat/Hello-World/releases/download/v1.0.0/example.zip",
        )
        self.assertEqual(asset.id, 1)
        self.assertEqual(asset.node_id, "MDEyOlJlbGVhc2VBc3NldDE=")
        self.assertEqual(asset.name, "example.zip")
        self.assertEqual(asset.label, "short description")
        self.assertEqual(asset.state, ReleaseAssetState.UPLOADED)
        self.assertEqual(asset.content_type, "application/zip")
        self.assertEqual(asset.size, 1024)
        self.assertEqual(asset.download_count, 42)
        self.assertEqual(
            asset.created_at,
            datetime(2013, 2, 27, 19, 35, 32, tzinfo=timezone.utc),
        )
        self.assertEqual(
            asset.updated_at,
            datetime(2013, 2, 27, 19, 35, 32, tzinfo=timezone.utc),
        )

        user = asset.uploader
        self.assertEqual(user.login, "octocat")
        self.assertEqual(user.id, 1)
        self.assertEqual(user.node_id, "MDQ6VXNlcjE=")
        self.assertEqual(
            user.avatar_url, "https://github.com/images/error/octocat_happy.gif"
        )
        self.assertEqual(user.gravatar_id, "")
        self.assertEqual(user.url, "https://api.github.com/users/octocat")
        self.assertEqual(user.html_url, "https://github.com/octocat")
        self.assertEqual(
            user.followers_url, "https://api.github.com/users/octocat/followers"
        )
        self.assertEqual(
            user.following_url,
            "https://api.github.com/users/octocat/following{/other_user}",
        )
        self.assertEqual(
            user.gists_url,
            "https://api.github.com/users/octocat/gists{/gist_id}",
        )
        self.assertEqual(
            user.starred_url,
            "https://api.github.com/users/octocat/starred{/owner}{/repo}",
        )
        self.assertEqual(
            user.subscriptions_url,
            "https://api.github.com/users/octocat/subscriptions",
        )
        self.assertEqual(
            user.organizations_url, "https://api.github.com/users/octocat/orgs"
        )
        self.assertEqual(
            user.repos_url, "https://api.github.com/users/octocat/repos"
        )
        self.assertEqual(
            user.events_url,
            "https://api.github.com/users/octocat/events{/privacy}",
        )
        self.assertEqual(
            user.received_events_url,
            "https://api.github.com/users/octocat/received_events",
        )
        self.assertEqual(user.type, "User")
        self.assertEqual(user.site_admin, False)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from argparse import ArgumentParser, FileType, Namespace
from datetime import datetime
from typing import Optional, Sequence

import shtab

SUPPORTED_LICENSES = [
    "AGPL-3.0-or-later",
    "GPL-2.0-only",
    "GPL-2.0-or-later",
    "GPL-3.0-or-later",
]


def parse_args(args: Optional[Sequence[str]] = None) -> Namespace:
    """Parsing the args"""

    parser = ArgumentParser(
        description="Update copyright in source file headers.",
    )
    shtab.add_argument_to(parser)

    parser.add_argument(
        "--quiet",
        "-q",
        action="store_true",
        help="Don't print messages to the terminal",
    )

    parser.add_argument(
        "--log-file",
        dest="log_file",
        type=str,
        help="Activate logging using the given file path",
    ).complete = shtab.FILE  # type: ignore[attr-defined]

    parser.add_argument(
        "-c",
        "--changed",
        action="store_true",
        default=False,
        help=(
            "Update modified year using git log modified year. "
            "Used instead of --year. If the modified year could not be "
            "determined via git it falls back to --year."
        ),
    )
    parser.add_argument(
        "-y",
        "--year",
        default=str(datetime.now().year),
        help=(
            "If year is set, modified year will be "
            "set to the specified year. Default is %(default)s."
        ),
    )

    parser.add_argument(
        "-l",
        "--license",
        dest="license_id",
        choices=SUPPORTED_LICENSES,
        default="GPL-3.0-or-later",
        help="Use the passed license type. Default is %(default)s",
    )

    parser.add_argument(
        "--company",
        default="Greenbone AG",
        help=(
            "If a header will be added to file, "
            "it will be licensed by company. Default is %(default)s"
        ),
    )

    files_group = parser.add_mutually_exclusive_group(required=True)
    files_group.add_argument(
        "-f", "--files", nargs="+", help="Files to update."
    ).complete = shtab.FILE  # type: ignore[attr-defined]
    files_group.add_argument(
        "-d",
        "--directories",
        nargs="+",
        help="Directories to find files to update recursively.",
    ).complete = shtab.DIRECTORY  # type: ignore[attr-defined]

    parser.add_argument(
        "--exclude-file",
        help=(
            "File containing glob patterns for files to "
            "ignore when finding files to update in a directory. "
            "Will look for '.pontos-header-ignore' in the directory "
            "if none is given. "
            "The ignore file should only contain relative paths like *.py,"
            "not absolute as **/*.py"
        ),
        type=FileType("r"),
    ).complete = shtab.FILE  # type: ignore[attr-defined]

    parser.add_argument(
        "--cleanup",
        action="store_true",
        default=False,
        help="Do a cleanup: Remove lines from outdated header format",
    )

    parser.add_argument(
        "--single-year",
        action="store_true",
        default=False,
        help=(
            "If set, will format license headers in from-to year format "
            "into single (creation) year format. "
            "Default is %(default)s."
        ),
    )

    return parser.parse_args(args)



# SPDX-FileCopyrightText: 2020-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=protected-access

import unittest
from unittest.mock import MagicMock, patch

from pontos.testing import temp_directory, temp_file
from pontos.version import VersionError
from pontos.version.commands._cmake import (
    CMakeVersionCommand,
    CMakeVersionParser,
)
from pontos.version.schemes import PEP440VersioningScheme


class VerifyCMakeVersionCommandTestCase(unittest.TestCase):
    def test_verify_failure(self):
        with temp_file(
            "project(VERSION 1.2.3)\nset(PROJECT_DEV_VERSION 0)",
            name="CMakeLists.txt",
            change_into=True,
        ):
            cmake = CMakeVersionCommand(PEP440VersioningScheme)
            version = PEP440VersioningScheme.parse_version("2.3.4")

            with self.assertRaisesRegex(
                VersionError,
                "Provided version 2.3.4 does not match the "
                "current version 1.2.3.",
            ):
                cmake.verify_version(version)

    def test_verify_current(self):
        with temp_directory(
            change_into=True,
        ):
            cmake = CMakeVersionCommand(PEP440VersioningScheme)

            with self.assertRaisesRegex(
                VersionError, "^.*CMakeLists.txt not found."
            ):
                cmake.verify_version("current")

            with self.assertRaisesRegex(
                VersionError, "^.*CMakeLists.txt not found."
            ):
                cmake.verify_version(version=None)


class GetCurrentCMakeVersionCommandTestCase(unittest.TestCase):
    # pylint: disable=line-too-long
    @patch(
        "pontos.version.commands._cmake.CMakeVersionCommand.get_current_version",
        MagicMock(return_value=PEP440VersioningScheme.parse_version("21.4")),
    )
    def test_return_0_correct_version_on_verify(self):
        with temp_file(
            "",
            name="CMakeLists.txt",
            change_into=True,
        ):
            cmake = CMakeVersionCommand(PEP440VersioningScheme)
            version = PEP440VersioningScheme.parse_version("21.4")
            cmake.verify_version(version)

    def test_should_call_print_current_version_without_raising_exception(self):
        with temp_file(
            "project(VERSION 21)",
            name="CMakeLists.txt",
            change_into=True,
        ):
            cmake = CMakeVersionCommand(PEP440VersioningScheme)
            version = PEP440VersioningScheme.parse_version("21")
            self.assertEqual(cmake.get_current_version(), version)


class UpdateCMakeVersionCommandTestCase(unittest.TestCase):
    def test_update_version(self):
        with temp_file(
            "project(VERSION 21)\nset(PROJECT_DEV_VERSION 0)",
            name="CMakeLists.txt",
            change_into=True,
        ) as temp:
            cmake = CMakeVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("22")
            previous_version = PEP440VersioningScheme.parse_version("21")
            updated = cmake.update_version(new_version)

            self.assertEqual(
                "project(VERSION 22)\nset(PROJECT_DEV_VERSION 0)",
                temp.read_text(encoding="utf8"),
            )
            self.assertEqual(updated.previous, previous_version)
            self.assertEqual(updated.new, new_version)
            self.assertEqual(updated.changed_files, [temp.resolve()])

    def test_update_dev_version(self):
        with temp_file(
            "project(VERSION 21)\nset(PROJECT_DEV_VERSION 0)",
            name="CMakeLists.txt",
            change_into=True,
        ) as temp:
            cmake = CMakeVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("22.dev1")
            previous_version = PEP440VersioningScheme.parse_version("21")
            updated = cmake.update_version(new_version)

            self.assertEqual(
                "project(VERSION 22.0.0)\nset(PROJECT_DEV_VERSION 1)",
                temp.read_text(encoding="utf8"),
            )
            self.assertEqual(updated.previous, previous_version)
            self.assertEqual(updated.new, new_version)
            self.assertEqual(updated.changed_files, [temp.resolve()])

    def test_no_update(self):
        with temp_file(
            "project(VERSION 22)\nset(PROJECT_DEV_VERSION 0)",
            name="CMakeLists.txt",
            change_into=True,
        ) as temp:
            cmake = CMakeVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("22")
            previous_version = PEP440VersioningScheme.parse_version("22")
            updated = cmake.update_version(new_version)

            self.assertEqual(
                "project(VERSION 22)\nset(PROJECT_DEV_VERSION 0)",
                temp.read_text(encoding="utf8"),
            )
            self.assertEqual(updated.previous, previous_version)
            self.assertEqual(updated.new, new_version)
            self.assertEqual(updated.changed_files, [])

    def test_forced_update(self):
        with temp_file(
            "project(VERSION 22)\nset(PROJECT_DEV_VERSION 0)",
            name="CMakeLists.txt",
            change_into=True,
        ) as temp:
            cmake = CMakeVersionCommand(PEP440VersioningScheme)
            new_version = PEP440VersioningScheme.parse_version("22")
            previous_version = PEP440VersioningScheme.parse_version("22")
            updated = cmake.update_version(new_version, force=True)

            self.assertEqual(
                "project(VERSION 22)\nset(PROJECT_DEV_VERSION 0)",
                temp.read_text(encoding="utf8"),
            )
            self.assertEqual(updated.previous, previous_version)
            self.assertEqual(updated.new, new_version)
            self.assertEqual(updated.changed_files, [temp.resolve()])


class ProjectFileCMakeVersionCommandTestCase(unittest.TestCase):
    def test_project_file_not_found(self):
        with temp_directory(change_into=True):
            cmd = CMakeVersionCommand(PEP440VersioningScheme)

            self.assertFalse(cmd.project_found())

    def test_project_file_found(self):
        with temp_file(name="CMakeLists.txt", change_into=True):
            cmd = CMakeVersionCommand(PEP440VersioningScheme)

            self.assertTrue(cmd.project_found())


class CMakeVersionParserTestCase(unittest.TestCase):
    def test_get_current_version_single_line_project(self):
        under_test = CMakeVersionParser("project(VERSION 2.3.4)")
        version = PEP440VersioningScheme.parse_version("2.3.4")
        self.assertEqual(under_test.get_current_version(), version)

    def test_update_version_project(self):
        under_test = CMakeVersionParser("project(VERSION 2.3.4)")
        version = PEP440VersioningScheme.parse_version("2.3.5")
        self.assertEqual(
            under_test.update_version(version),
            "project(VERSION 2.3.5)",
        )

    def test_not_confuse_version_outside_project(self):
        under_test = CMakeVersionParser(
            "non_project(VERSION 2.3.5)\nproject(VERSION 2.3.4)"
        )
        version = PEP440VersioningScheme.parse_version("2.3.4")
        self.assertEqual(under_test.get_current_version(), version)

    def test_get_current_version_multiline_project(self):
        under_test = CMakeVersionParser("project\n(\nVERSION\n\t    2.3.4)")
        version = PEP440VersioningScheme.parse_version("2.3.4")
        self.assertEqual(under_test.get_current_version(), version)

    def test_find_project_dev_version(self):
        test_cmake_lists = """
        project(
            DESCRIPTION something
            VERSION 41.41.41
            LANGUAGES c
        )
        set(
            PROJECT_DEV_VERSION 1
        )
        """
        under_test = CMakeVersionParser(test_cmake_lists)
        self.assertEqual(under_test._project_dev_version_line_number, 7)
        self.assertEqual(under_test._project_dev_version, "1")

    def test_update_project_dev_version(self):
        test_cmake_lists = """
        project(
            DESCRIPTION something
            VERSION 41.41.41
            LANGUAGES c
        )
        set(
            PROJECT_DEV_VERSION 1
        )
        """
        under_test = CMakeVersionParser(test_cmake_lists)
        version = PEP440VersioningScheme.parse_version("41.41.41")

        self.assertEqual(under_test._project_dev_version_line_number, 7)
        self.assertEqual(under_test._project_dev_version, "1")

        result = under_test.update_version(version)

        self.assertEqual(under_test._project_dev_version, "0")
        self.assertEqual(
            result,
            test_cmake_lists.replace(
                "PROJECT_DEV_VERSION 1", "PROJECT_DEV_VERSION 0"
            ),
        )

    def test_update_project_dev_version_when_succeeded_by_another_set(self):
        test_cmake_lists = """
        cmake_minimum_required(VERSION 3.1)

        project(hello_world VERSION 41.41.41)
        set(PROJECT_DEV_VERSION 1)

        add_executable(app main.c)
        """
        under_test = CMakeVersionParser(test_cmake_lists)
        version = PEP440VersioningScheme.parse_version("41.41.41")

        self.assertEqual(under_test._project_dev_version_line_number, 4)
        self.assertEqual(under_test._project_dev_version, "1")

        result = under_test.update_version(version)

        self.assertEqual(under_test._project_dev_version, "0")
        self.assertEqual(
            result,
            test_cmake_lists.replace(
                "PROJECT_DEV_VERSION 1", "PROJECT_DEV_VERSION 0"
            ),
        )

    def test_get_current_version_multiline_project_combined_token(self):
        under_test = CMakeVersionParser(
            "project\n(\nDESCRIPTION something VERSION 2.3.4 LANGUAGES c\n)"
        )
        version = PEP440VersioningScheme.parse_version("2.3.4")
        self.assertEqual(under_test.get_current_version(), version)

    def test_raise_exception_project_no_version(self):
        with self.assertRaises(ValueError) as context:
            CMakeVersionParser("project(DESCRIPTION something LANGUAGES c)")
        self.assertEqual(
            str(context.exception), "unable to find cmake version in project."
        )

    def test_raise_exception_no_project(self):
        with self.assertRaises(ValueError) as context:
            CMakeVersionParser(
                "non_project(VERSION 2.3.5)",
            )

        self.assertEqual(
            str(context.exception), "unable to find cmake version."
        )



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .api import NVDApi, NVDResults, convert_camel_case, format_date, now

__all__ = (
    "convert_camel_case",
    "format_date",
    "now",
    "NVDApi",
    "NVDResults",
)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import ArgumentTypeError
from enum import Enum
from typing import Callable, Type, TypeVar, Union


class StrEnum(str, Enum):
    # Should be replaced by enum.StrEnum when we require Python >= 3.11
    """
    An Enum that provides str like behavior
    """

    def __str__(self) -> str:
        return self.value


def enum_choice(enum: Type[Enum]) -> list[str]:
    """
    Return a sequence of choices for argparse from an enum
    """
    return [str(e) for e in enum]


def to_choices(enum: Type[Enum]) -> str:
    """
    Convert an enum to a comma separated string of choices. For example useful
    in help messages for argparse.
    """
    return ", ".join([str(t) for t in enum])


T = TypeVar("T", bound=Enum)


def enum_type(enum: Type[T]) -> Callable[[Union[str, T]], T]:
    """
    Create a argparse type function for converting the string input into an Enum
    """

    def convert(value: Union[str, T]) -> T:
        if isinstance(value, str):
            try:
                return enum(value)
            except ValueError:
                raise ArgumentTypeError(
                    f"invalid value {value}. Expected one of {to_choices(enum)}."
                ) from None
        return value

    return convert



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=too-many-lines, redefined-builtin, line-too-long

from copy import deepcopy
from pathlib import Path
from unittest.mock import MagicMock

import httpx

from pontos.github.api.errors import GitHubApiError
from pontos.github.api.organizations import (
    GitHubAsyncRESTOrganizations,
    InvitationRole,
    MemberFilter,
    MemberRole,
)
from pontos.github.models.organization import RepositoryType
from tests import AsyncIteratorMock, aiter, anext
from tests.github.api import GitHubAsyncRESTTestCase, create_response

here = Path(__file__).parent

REPOSITORY_DICT = {
    "id": 1,
    "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
    "name": "Hello-World",
    "full_name": "octocat/Hello-World",
    "owner": {
        "login": "octocat",
        "id": 1,
        "node_id": "MDQ6VXNlcjE=",
        "avatar_url": "https://github.com/images/error/octocat_happy.gif",
        "gravatar_id": "",
        "url": "https://api.github.com/users/octocat",
        "html_url": "https://github.com/octocat",
        "followers_url": "https://api.github.com/users/octocat/followers",
        "following_url": "https://api.github.com/users/octocat/following{/other_user}",
        "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
        "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
        "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
        "organizations_url": "https://api.github.com/users/octocat/orgs",
        "repos_url": "https://api.github.com/users/octocat/repos",
        "events_url": "https://api.github.com/users/octocat/events{/privacy}",
        "received_events_url": "https://api.github.com/users/octocat/received_events",
        "type": "User",
        "site_admin": False,
    },
    "private": False,
    "html_url": "https://github.com/octocat/Hello-World",
    "description": "This your first repo!",
    "fork": False,
    "url": "https://api.github.com/repos/octocat/Hello-World",
    "archive_url": "https://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
    "assignees_url": "https://api.github.com/repos/octocat/Hello-World/assignees{/user}",
    "blobs_url": "https://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
    "branches_url": "https://api.github.com/repos/octocat/Hello-World/branches{/branch}",
    "collaborators_url": "https://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
    "comments_url": "https://api.github.com/repos/octocat/Hello-World/comments{/number}",
    "commits_url": "https://api.github.com/repos/octocat/Hello-World/commits{/sha}",
    "compare_url": "https://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
    "contents_url": "https://api.github.com/repos/octocat/Hello-World/contents/{+path}",
    "contributors_url": "https://api.github.com/repos/octocat/Hello-World/contributors",
    "deployments_url": "https://api.github.com/repos/octocat/Hello-World/deployments",
    "downloads_url": "https://api.github.com/repos/octocat/Hello-World/downloads",
    "events_url": "https://api.github.com/repos/octocat/Hello-World/events",
    "forks_url": "https://api.github.com/repos/octocat/Hello-World/forks",
    "git_commits_url": "https://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
    "git_refs_url": "https://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
    "git_tags_url": "https://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
    "git_url": "git:github.com/octocat/Hello-World.git",
    "issue_comment_url": "https://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
    "issue_events_url": "https://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
    "issues_url": "https://api.github.com/repos/octocat/Hello-World/issues{/number}",
    "keys_url": "https://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
    "labels_url": "https://api.github.com/repos/octocat/Hello-World/labels{/name}",
    "languages_url": "https://api.github.com/repos/octocat/Hello-World/languages",
    "merges_url": "https://api.github.com/repos/octocat/Hello-World/merges",
    "milestones_url": "https://api.github.com/repos/octocat/Hello-World/milestones{/number}",
    "notifications_url": "https://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
    "pulls_url": "https://api.github.com/repos/octocat/Hello-World/pulls{/number}",
    "releases_url": "https://api.github.com/repos/octocat/Hello-World/releases{/id}",
    "ssh_url": "git@github.com:octocat/Hello-World.git",
    "stargazers_url": "https://api.github.com/repos/octocat/Hello-World/stargazers",
    "statuses_url": "https://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
    "subscribers_url": "https://api.github.com/repos/octocat/Hello-World/subscribers",
    "subscription_url": "https://api.github.com/repos/octocat/Hello-World/subscription",
    "tags_url": "https://api.github.com/repos/octocat/Hello-World/tags",
    "teams_url": "https://api.github.com/repos/octocat/Hello-World/teams",
    "trees_url": "https://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
    "clone_url": "https://github.com/octocat/Hello-World.git",
    "mirror_url": "git:git.example.com/octocat/Hello-World",
    "hooks_url": "https://api.github.com/repos/octocat/Hello-World/hooks",
    "svn_url": "https://svn.github.com/octocat/Hello-World",
    "homepage": "https://github.com",
    "language": None,
    "forks_count": 9,
    "stargazers_count": 80,
    "watchers_count": 80,
    "size": 108,
    "default_branch": "master",
    "open_issues_count": 0,
    "is_template": False,
    "topics": ["octocat", "atom", "electron", "api"],
    "has_issues": True,
    "has_projects": True,
    "has_wiki": True,
    "has_pages": False,
    "has_downloads": True,
    "has_discussions": False,
    "archived": False,
    "disabled": False,
    "visibility": "public",
    "pushed_at": "2011-01-26T19:06:43Z",
    "created_at": "2011-01-26T19:01:12Z",
    "updated_at": "2011-01-26T19:14:43Z",
    "permissions": {"admin": False, "push": False, "pull": True},
    "forks": 1,
    "open_issues": 0,
    "watchers": 1,
}

MEMBER_DICT = {
    "id": 1,
    "login": "octocat",
    "node_id": "MDQ6VXNlcjE=",
    "avatar_url": "https://github.com/images/error/octocat_happy.gif",
    "gravatar_id": "",
    "url": "https://api.github.com/users/octocat",
    "html_url": "https://github.com/octocat",
    "followers_url": "https://api.github.com/users/octocat/followers",
    "following_url": "https://api.github.com/users/octocat/following{/other_user}",
    "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
    "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
    "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
    "organizations_url": "https://api.github.com/users/octocat/orgs",
    "repos_url": "https://api.github.com/users/octocat/repos",
    "events_url": "https://api.github.com/users/octocat/events{/privacy}",
    "received_events_url": "https://api.github.com/users/octocat/received_events",
    "type": "User",
    "site_admin": False,
}


class GitHubAsyncRESTOrganizationsTestCase(GitHubAsyncRESTTestCase):
    api_cls = GitHubAsyncRESTOrganizations

    async def test_exists(self):
        response = create_response(is_success=True)
        self.client.get.return_value = response

        self.assertTrue(await self.api.exists("foo"))

        self.client.get.assert_awaited_once_with("/orgs/foo")

    async def test_not_exists(self):
        response = create_response(is_success=False)
        self.client.get.return_value = response

        self.assertFalse(await self.api.exists("foo"))

        self.client.get.assert_awaited_once_with("/orgs/foo")

    async def test_get_repositories(self):
        response1 = create_response()
        response1.json.return_value = [REPOSITORY_DICT]
        response2 = create_response()
        repository_dict2 = deepcopy(REPOSITORY_DICT)
        repository_dict2["id"] = 2
        repository_dict3 = deepcopy(REPOSITORY_DICT)
        repository_dict3["id"] = 3
        response2.json.return_value = [repository_dict2, repository_dict3]

        self.client.get_all.return_value = AsyncIteratorMock(
            [response1, response2]
        )

        async_it = aiter(self.api.get_repositories("foo"))
        repo = await anext(async_it)
        self.assertEqual(repo.id, 1)
        repo = await anext(async_it)
        self.assertEqual(repo.id, 2)
        repo = await anext(async_it)
        self.assertEqual(repo.id, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/repos",
            params={"per_page": "100", "type": "all"},
        )

    async def test_get_private_repositories(self):
        response1 = create_response()
        response1.json.return_value = [REPOSITORY_DICT]
        response2 = create_response()
        repository_dict2 = deepcopy(REPOSITORY_DICT)
        repository_dict2["id"] = 2
        repository_dict3 = deepcopy(REPOSITORY_DICT)
        repository_dict3["id"] = 3
        response2.json.return_value = [repository_dict2, repository_dict3]

        self.client.get_all.return_value = AsyncIteratorMock(
            [response1, response2]
        )

        async_it = aiter(
            self.api.get_repositories(
                "foo", repository_type=RepositoryType.PRIVATE
            )
        )
        repo = await anext(async_it)
        self.assertEqual(repo.id, 1)
        repo = await anext(async_it)
        self.assertEqual(repo.id, 2)
        repo = await anext(async_it)
        self.assertEqual(repo.id, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/repos",
            params={"per_page": "100", "type": "private"},
        )

    async def test_members(self):
        response1 = create_response()
        response1.json.return_value = [MEMBER_DICT]
        response2 = create_response()
        member_dict2 = deepcopy(MEMBER_DICT)
        member_dict2["id"] = 2
        member_dict3 = deepcopy(MEMBER_DICT)
        member_dict3["id"] = 3
        response2.json.return_value = [member_dict2, member_dict3]

        self.client.get_all.return_value = AsyncIteratorMock(
            [response1, response2]
        )

        async_it = aiter(self.api.members("foo"))
        member = await anext(async_it)
        self.assertEqual(member.id, 1)
        member = await anext(async_it)
        self.assertEqual(member.id, 2)
        member = await anext(async_it)
        self.assertEqual(member.id, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/members",
            params={"per_page": "100", "filter": "all", "role": "all"},
        )

    async def test_members_admins(self):
        response1 = create_response()
        response1.json.return_value = [MEMBER_DICT]
        response2 = create_response()
        member_dict2 = deepcopy(MEMBER_DICT)
        member_dict2["id"] = 2
        member_dict3 = deepcopy(MEMBER_DICT)
        member_dict3["id"] = 3
        response2.json.return_value = [member_dict2, member_dict3]

        self.client.get_all.return_value = AsyncIteratorMock(
            [response1, response2]
        )

        async_it = aiter(self.api.members("foo", role=MemberRole.ADMIN))
        member = await anext(async_it)
        self.assertEqual(member.id, 1)
        member = await anext(async_it)
        self.assertEqual(member.id, 2)
        member = await anext(async_it)
        self.assertEqual(member.id, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/members",
            params={"per_page": "100", "filter": "all", "role": "admin"},
        )

    async def test_members_filter(self):
        response1 = create_response()
        response1.json.return_value = [MEMBER_DICT]
        response2 = create_response()
        member_dict2 = deepcopy(MEMBER_DICT)
        member_dict2["id"] = 2
        member_dict3 = deepcopy(MEMBER_DICT)
        member_dict3["id"] = 3
        response2.json.return_value = [member_dict2, member_dict3]

        self.client.get_all.return_value = AsyncIteratorMock(
            [response1, response2]
        )

        async_it = aiter(
            self.api.members("foo", member_filter=MemberFilter.TWO_FA_DISABLED)
        )
        member = await anext(async_it)
        self.assertEqual(member.id, 1)
        member = await anext(async_it)
        self.assertEqual(member.id, 2)
        member = await anext(async_it)
        self.assertEqual(member.id, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/members",
            params={"per_page": "100", "filter": "2fa_disabled", "role": "all"},
        )

    async def test_invite_email(self):
        response = create_response(is_success=False)
        self.client.post.return_value = response

        await self.api.invite(
            "foo",
            email="foo@bar.com",
        )

        self.client.post.assert_awaited_once_with(
            "/orgs/foo/invitations",
            data={"role": "direct_member", "email": "foo@bar.com"},
        )

    async def test_invite_invitee(self):
        response = create_response(is_success=False)
        self.client.post.return_value = response

        await self.api.invite(
            "foo",
            invitee_id="foo",
        )

        self.client.post.assert_awaited_once_with(
            "/orgs/foo/invitations",
            data={"role": "direct_member", "invitee_id": "foo"},
        )

    async def test_invite_missing_user(self):
        response = create_response(is_success=False)
        self.client.post.return_value = response

        with self.assertRaises(GitHubApiError):
            await self.api.invite("foo")

    async def test_invite_with_teams(self):
        response = create_response(is_success=False)
        self.client.post.return_value = response

        await self.api.invite("foo", email="foo@bar.com", team_ids=("1", "2"))

        self.client.post.assert_awaited_once_with(
            "/orgs/foo/invitations",
            data={
                "role": "direct_member",
                "email": "foo@bar.com",
                "team_ids": ["1", "2"],
            },
        )

    async def test_invite_with_role(self):
        response = create_response(is_success=False)
        self.client.post.return_value = response

        await self.api.invite(
            "foo", email="foo@bar.com", role=InvitationRole.ADMIN
        )

        self.client.post.assert_awaited_once_with(
            "/orgs/foo/invitations",
            data={
                "role": "admin",
                "email": "foo@bar.com",
            },
        )

    async def test_remove_member(self):
        response = create_response(is_success=False)
        self.client.delete.return_value = response

        await self.api.remove_member("foo", "bar")

        self.client.delete.assert_awaited_once_with(
            "/orgs/foo/memberships/bar",
        )

    async def test_remove_member_failure(self):
        response = create_response()
        self.client.delete.side_effect = httpx.HTTPStatusError(
            "404", request=MagicMock(), response=response
        )

        with self.assertRaises(httpx.HTTPStatusError):
            await self.api.remove_member("foo", "bar")

        self.client.delete.assert_awaited_once_with(
            "/orgs/foo/memberships/bar",
        )

    async def test_outside_collaborators(self):
        response1 = create_response()
        response1.json.return_value = [MEMBER_DICT]
        response2 = create_response()
        member_dict2 = deepcopy(MEMBER_DICT)
        member_dict2["id"] = 2
        member_dict3 = deepcopy(MEMBER_DICT)
        member_dict3["id"] = 3
        response2.json.return_value = [member_dict2, member_dict3]

        self.client.get_all.return_value = AsyncIteratorMock(
            [response1, response2]
        )

        async_it = aiter(self.api.outside_collaborators("foo"))
        member = await anext(async_it)
        self.assertEqual(member.id, 1)
        member = await anext(async_it)
        self.assertEqual(member.id, 2)
        member = await anext(async_it)
        self.assertEqual(member.id, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/outside_collaborators",
            params={"per_page": "100", "filter": "all"},
        )

    async def test_outside_collaborators_filter(self):
        response1 = create_response()
        response1.json.return_value = [MEMBER_DICT]
        response2 = create_response()
        member_dict2 = deepcopy(MEMBER_DICT)
        member_dict2["id"] = 2
        member_dict3 = deepcopy(MEMBER_DICT)
        member_dict3["id"] = 3
        response2.json.return_value = [member_dict2, member_dict3]

        self.client.get_all.return_value = AsyncIteratorMock(
            [response1, response2]
        )

        async_it = aiter(
            self.api.outside_collaborators(
                "foo", member_filter=MemberFilter.TWO_FA_DISABLED
            )
        )
        member = await anext(async_it)
        self.assertEqual(member.id, 1)
        member = await anext(async_it)
        self.assertEqual(member.id, 2)
        member = await anext(async_it)
        self.assertEqual(member.id, 3)

        with self.assertRaises(StopAsyncIteration):
            await anext(async_it)

        self.client.get_all.assert_called_once_with(
            "/orgs/foo/outside_collaborators",
            params={"per_page": "100", "filter": "2fa_disabled"},
        )

    async def test_remove_outside_collaborator(self):
        response = create_response(is_success=False)
        self.client.delete.return_value = response

        await self.api.remove_outside_collaborator("foo", "bar")

        self.client.delete.assert_awaited_once_with(
            "/orgs/foo/outside_collaborators/bar",
        )

    async def test_remove_outside_collaborator_failure(self):
        response = create_response()
        self.client.delete.side_effect = httpx.HTTPStatusError(
            "404", request=MagicMock(), response=response
        )

        with self.assertRaises(httpx.HTTPStatusError):
            await self.api.remove_outside_collaborator("foo", "bar")

        self.client.delete.assert_awaited_once_with(
            "/orgs/foo/outside_collaborators/bar",
        )



# SPDX-FileCopyrightText: 2021-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from pathlib import Path

from pontos.github.actions.event import GitHubEvent, PullRequestState

here = Path(__file__).parent


class GitHubPullRequestEventTestCase(unittest.TestCase):
    def setUp(self) -> None:
        event_path = Path(here / "test-pull-request-event.json")
        event = GitHubEvent(event_path)
        self.pull_request = event.pull_request

    def test_draft(self):
        self.assertFalse(self.pull_request.draft)

    def test_labels(self):
        self.assertEqual(len(self.pull_request.labels), 1)
        self.assertEqual(self.pull_request.labels[0].name, "enhancement")

    def test_number(self):
        self.assertEqual(self.pull_request.number, 1)

    def test_title(self):
        self.assertEqual(self.pull_request.title, "Add foo for bar")

    def test_state(self):
        self.assertEqual(self.pull_request.state, PullRequestState.OPEN)

    def test_base(self):
        base = self.pull_request.base
        self.assertEqual(base.name, "main")
        self.assertEqual(base.sha, "50c0fbe90d0c19dba08165b707e8b720d604ed5d")

    def test_head(self):
        head = self.pull_request.head
        self.assertEqual(head.name, "label-test")
        self.assertEqual(head.sha, "beeecfeee02f5a9e69c1f69dba6757df6e5c20f7")

    def test_merged(self):
        self.assertEqual(self.pull_request.merged, False)



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import sys
from argparse import Namespace
from pathlib import Path

import httpx

from pontos.github.api import GitHubAsyncRESTApi
from pontos.terminal import Terminal


async def tag(terminal: Terminal, args: Namespace) -> None:
    """Github release function for argument class to call"""
    await args.tag_func(terminal, args)


async def create_tag(terminal: Terminal, args: Namespace) -> None:
    """Github create tag function for
    argument class to call"""
    async with GitHubAsyncRESTApi(token=args.token) as api:
        try:
            # Create tag
            new_tag = await api.tags.create(
                repo=args.repo,
                tag=args.tag,
                message=args.message,
                git_object=args.git_object,
                name=args.name,
                email=args.email,
                git_object_type=args.git_object_type,
                date=args.date,
            )

            # Create tag reference
            await api.tags.create_tag_reference(
                repo=args.repo, tag=args.tag, sha=new_tag.sha
            )

        except httpx.HTTPError as e:
            terminal.error(str(e))
            sys.exit(1)

        terminal.ok("Tag created.")


async def release(terminal: Terminal, args: Namespace) -> None:
    """Github release function for argument class to call"""
    await args.re_func(terminal, args)


async def create_release(terminal: Terminal, args: Namespace) -> None:
    """Github create release function for
    argument class to call"""

    async with GitHubAsyncRESTApi(token=args.token) as api:
        try:
            # Check if release exist
            exists = await api.releases.exists(repo=args.repo, tag=args.tag)
            if exists:
                terminal.error(f"Release {args.tag} exist.")
                sys.exit(1)

            # Create release
            await api.releases.create(
                repo=args.repo,
                tag=args.tag,
                body=args.body,
                name=args.name,
                target_commitish=args.target_commitish,
                draft=args.draft,
                prerelease=args.prerelease,
            )

            terminal.ok("Release created.")
        except httpx.HTTPError as e:
            terminal.error(str(e))
            sys.exit(1)


async def pull_request(terminal: Terminal, args: Namespace):
    await args.pr_func(terminal, args)


async def create_pull_request(terminal: Terminal, args: Namespace):
    async with GitHubAsyncRESTApi(token=args.token) as api:
        try:
            # check if branches exist
            exists = await api.branches.exists(repo=args.repo, branch=args.head)
            if not exists:
                terminal.error(
                    f"Head branch {args.head} is not existing "
                    "or authorisation failed."
                )
                sys.exit(1)

            terminal.ok(f"Head branch {args.head} is existing.")

            exists = await api.branches.exists(
                repo=args.repo, branch=args.target
            )
            if not exists:
                terminal.error(
                    f"Target branch {args.target} is not existing or "
                    "authorisation failed."
                )
                sys.exit(1)

            terminal.ok(f"Target branch {args.target} exists.")

            await api.pull_requests.create(
                repo=args.repo,
                head_branch=args.head,
                base_branch=args.target,
                title=args.title,
                body=args.body,
            )

            terminal.ok("Pull Request created.")
        except httpx.HTTPError as e:
            terminal.error(str(e))
            sys.exit(1)


async def update_pull_request(terminal: Terminal, args: Namespace):
    async with GitHubAsyncRESTApi(token=args.token) as api:
        try:
            if args.target:
                # check if branches exist
                exists = await api.branches.exists(
                    repo=args.repo, branch=args.target
                )
                if not exists:
                    terminal.error(
                        f"Target branch {args.target} is not existing or "
                        "authorisation failed."
                    )
                    sys.exit(1)

                terminal.ok(f"Target branch {args.target} exists.")

            await api.pull_requests.update(
                repo=args.repo,
                pull_request=args.pull_request,
                base_branch=args.target,
                title=args.title,
                body=args.body,
            )

            terminal.ok("Pull Request updated.")
        except httpx.HTTPError as e:
            terminal.error(str(e))
            sys.exit(1)


async def file_status(terminal: Terminal, args: Namespace):
    async with GitHubAsyncRESTApi(token=args.token) as api:
        try:
            # check if PR is existing
            exists = await api.pull_requests.exists(
                repo=args.repo, pull_request=args.pull_request
            )
            if not exists:
                terminal.error(
                    f"PR {args.pull_request} is not existing "
                    "or authorisation failed."
                )
                sys.exit(1)

            terminal.ok(f"PR {args.pull_request} exists.")

            file_dict = await api.pull_requests.files(
                repo=args.repo,
                pull_request=args.pull_request,
                status_list=args.status,
            )
            for status in args.status:
                terminal.info(f"{status.value}:")
                files = [str(f.resolve()) for f in file_dict[status]]

                for file_string in files:
                    terminal.print(file_string)

                if args.output:
                    args.output.write("\n".join(files) + "\n")

        except httpx.HTTPError as e:
            terminal.error(str(e))
            sys.exit(1)


async def labels(terminal: Terminal, args: Namespace):
    async with GitHubAsyncRESTApi(token=args.token) as api:
        try:
            # check if PR is existing
            exists = await api.pull_requests.exists(
                repo=args.repo, pull_request=args.issue
            )
            if not exists:
                terminal.error(
                    f"PR {args.issue} is not existing or authorisation failed."
                )
                sys.exit(1)

            terminal.ok(f"PR {args.issue} exists.")

            issue_labels = []
            async for label in api.labels.get_all(
                repo=args.repo,
                issue=args.issue,
            ):
                issue_labels.append(label)

            issue_labels.extend(args.labels)

            await api.labels.set_all(
                repo=args.repo, issue=args.issue, labels=issue_labels
            )
        except httpx.HTTPError as e:
            terminal.error(str(e))
            sys.exit(1)


async def repos(terminal: Terminal, args: Namespace):
    async with GitHubAsyncRESTApi(token=args.token) as api:
        try:
            # check if Orga is existing
            exists = await api.organizations.exists(args.orga)
            if not exists:
                terminal.error(
                    f"Organization {args.orga} is not existing or authorisation failed."
                )
                sys.exit(1)

            terminal.ok(f"Organization {args.orga} exists.")

            if args.path:
                repo_info = Path(args.path)
                with repo_info.open(encoding="utf-8", mode="w") as fp:
                    async for repo in api.organizations.get_repositories(
                        organization=args.orga, repository_type=args.type
                    ):
                        fp.write(repr(repo))

            else:
                async for repo in api.organizations.get_repositories(
                    organization=args.orga, repository_type=args.type
                ):
                    terminal.print(repo)

        except httpx.HTTPError as e:
            terminal.error(str(e))
            sys.exit(1)



# SPDX-FileCopyrightText: 2020-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .__version__ import __version__
from ._calculator import VersionCalculator
from ._errors import VersionError
from ._main import main
from ._version import ParseVersionFuncType, Version, VersionUpdate

__all__ = (
    "__version__",
    "VersionError",
    "ParseVersionFuncType",
    "Version",
    "VersionCalculator",
    "VersionUpdate",
    "main",
)



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from dataclasses import dataclass
from datetime import date, datetime, timezone
from inspect import isclass
from typing import Any, Dict, Type, Union, get_args, get_origin, get_type_hints

from dateutil import parser as dateparser

from pontos.enum import StrEnum
from pontos.errors import PontosError

__all__ = (
    "Model",
    "ModelError",
    "StrEnum",
    "dotted_attributes",
)


class ModelError(PontosError):
    """
    Errors raised for Models
    """


def dotted_attributes(obj: Any, data: Dict[str, Any]) -> Any:
    """
    Set dotted attributes on an object

    Example:
        .. code-block:: python

            class Foo:
                '''Some class'''

            foo = Foo()
            attrs = {"bar": 123, "baz": 456}

            foo = dotted_attributes(foo, attrs)
            print(foo.bar, foo.baz)
    """
    for key, value in data.items():
        if isinstance(value, dict):
            default = None if hasattr(obj, key) else ModelAttribute()
            prop = getattr(obj, key, default)
            value = dotted_attributes(prop, value)  # noqa: PLW2901

        setattr(obj, key, value)

    return obj


class ModelAttribute:
    """
    A utility class to allow setting attributes
    """


def _get_value_from_model_field_cls(
    model_field_cls: Type[Any], value: Any
) -> Any:
    if isclass(model_field_cls) and issubclass(model_field_cls, Model):
        value = model_field_cls.from_dict(value)
    elif isclass(model_field_cls) and issubclass(model_field_cls, datetime):
        # Only Python 3.11 supports sufficient formats in
        # datetime.fromisoformat. Therefore we have to use dateutil here.
        value = dateparser.isoparse(value)
        # the iso format may not contain UTC data or a UTC offset
        # this means it is considered local time (Python calls this "naive"
        # datetime) and can't really be compared to other times.
        # Let's UTC in these cases:
        if not value.tzinfo:
            value = value.replace(tzinfo=timezone.utc)
    elif isclass(model_field_cls) and issubclass(model_field_cls, date):
        value = date.fromisoformat(value)
    elif get_origin(model_field_cls) is list:
        model_field_cls = get_args(model_field_cls)[0]
        value = _get_value_from_model_field_cls(model_field_cls, value)
    elif get_origin(model_field_cls) is dict:
        model_field_cls = dict
        value = _get_value_from_model_field_cls(model_field_cls, value)
    elif get_origin(model_field_cls) is Union:
        possible_types = get_args(model_field_cls)
        current_type = type(value)
        if current_type in possible_types:
            model_field_cls = current_type
        else:
            # currently Unions should not contain Models. this would require
            # to iterate over the possible type, check if it is a Model
            # class and try to create an instance of this class until it
            # fits. For now just fallback to first type
            model_field_cls = possible_types[0]

        value = _get_value_from_model_field_cls(model_field_cls, value)
    else:
        if isinstance(value, dict):
            value = model_field_cls(**value)
        else:
            value = model_field_cls(value)
    return value


def _get_value(model_field_cls: Type[Any], value: Any) -> Any:
    if model_field_cls:
        value = _get_value_from_model_field_cls(model_field_cls, value)
    return value


@dataclass(init=False)
class Model:
    """
    Base class for models
    """

    @classmethod
    def from_dict(cls, data: Dict[str, Any]):
        """
        Create a model from a dict

        Example:
            .. code-block:: python

                model = Model.from_dict({
                    "id": 123,
                    "node_id": "abcde",
                    "created_at": "2017-07-08T16:18:44-04:00",
                    "updated_at": "2017-07-08T16:18:44-04:00",
                })
        """
        if not isinstance(data, dict):
            raise ValueError(
                f"Invalid data for creating an instance of {cls.__name__} "
                f"model. Data is {data!r}"
            )

        kwargs = {}
        additional_attrs = {}
        type_hints = get_type_hints(cls)
        for name, value in data.items():
            try:
                if isinstance(value, list):
                    model_field_cls = type_hints.get(name)
                    value = [_get_value(model_field_cls, v) for v in value]  # type: ignore # pylint: disable=line-too-long # noqa: E501,PLW2901
                elif value is not None:
                    model_field_cls = type_hints.get(name)
                    value = _get_value(model_field_cls, value)  # type: ignore # pylint: disable=line-too-long # noqa: E501,PLW2901
            except (ValueError, TypeError) as e:
                raise ModelError(
                    f"Error while creating {cls.__name__} model. Could not set "
                    f"value for property '{name}' from '{value}'."
                ) from e

            if name in type_hints:
                kwargs[name] = value
            else:
                additional_attrs[name] = value

        instance = cls(**kwargs)
        dotted_attributes(instance, additional_attrs)
        return instance



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# ruff: noqa:E501

import unittest
from datetime import datetime, timezone

from pontos.github.models.dependabot import (
    AlertState,
    DependabotAlert,
    DependencyScope,
    DismissedReason,
    IdentifierType,
    SecurityAdvisory,
    Severity,
    Vulnerability,
    VulnerablePackage,
)


class VulnerablePackageTestCase(unittest.TestCase):
    def test_from_dict(self):
        package = VulnerablePackage.from_dict(
            {
                "ecosystem": "pip",
                "name": "django",
            }
        )

        self.assertEqual(package.ecosystem, "pip")
        self.assertEqual(package.name, "django")


class VulnerabilityTestCase(unittest.TestCase):
    def test_from_dict(self):
        vulnerability = Vulnerability.from_dict(
            {
                "package": {
                    "ecosystem": "pip",
                    "name": "django",
                },
                "severity": "high",
                "vulnerable_version_range": ">= 2.0.0, < 2.0.2",
                "first_patched_version": {"identifier": "2.0.2"},
            }
        )

        self.assertEqual(vulnerability.package.ecosystem, "pip")
        self.assertEqual(vulnerability.package.name, "django")
        self.assertEqual(vulnerability.severity, Severity.HIGH)
        self.assertEqual(
            vulnerability.vulnerable_version_range, ">= 2.0.0, < 2.0.2"
        )
        self.assertEqual(
            vulnerability.first_patched_version.identifier, "2.0.2"
        )

        vulnerability = Vulnerability.from_dict(
            {
                "package": {
                    "ecosystem": "pip",
                    "name": "django",
                },
                "severity": "high",
                "vulnerable_version_range": ">= 2.0.0, < 2.0.2",
            }
        )

        self.assertEqual(vulnerability.package.ecosystem, "pip")
        self.assertEqual(vulnerability.package.name, "django")
        self.assertEqual(vulnerability.severity, Severity.HIGH)
        self.assertIsNone(vulnerability.first_patched_version)


SECURITY_ADVISORY = {
    "ghsa_id": "GHSA-rf4j-j272-fj86",
    "cve_id": "CVE-2018-6188",
    "summary": "Django allows remote attackers to obtain potentially sensitive information by leveraging data exposure from the confirm_login_allowed() method, as demonstrated by discovering whether a user account is inactive",
    "description": "django.contrib.auth.forms.AuthenticationForm in Django 2.0 before 2.0.2, and 1.11.8 and 1.11.9, allows remote attackers to obtain potentially sensitive information by leveraging data exposure from the confirm_login_allowed() method, as demonstrated by discovering whether a user account is inactive.",
    "vulnerabilities": [
        {
            "package": {"ecosystem": "pip", "name": "django"},
            "severity": "high",
            "vulnerable_version_range": ">= 2.0.0, < 2.0.2",
            "first_patched_version": {"identifier": "2.0.2"},
        },
        {
            "package": {"ecosystem": "pip", "name": "django"},
            "severity": "high",
            "vulnerable_version_range": ">= 1.11.8, < 1.11.10",
            "first_patched_version": {"identifier": "1.11.10"},
        },
    ],
    "severity": "high",
    "cvss": {
        "vector_string": "CVSS:3.0/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:N/A:N",
        "score": 7.5,
    },
    "cwes": [
        {
            "cwe_id": "CWE-200",
            "name": "Exposure of Sensitive Information to an Unauthorized Actor",
        }
    ],
    "identifiers": [
        {"type": "GHSA", "value": "GHSA-rf4j-j272-fj86"},
        {"type": "CVE", "value": "CVE-2018-6188"},
    ],
    "references": [
        {"url": "https://nvd.nist.gov/vuln/detail/CVE-2018-6188"},
        {"url": "https://github.com/advisories/GHSA-rf4j-j272-fj86"},
        {"url": "https://usn.ubuntu.com/3559-1/"},
        {
            "url": "https://www.djangoproject.com/weblog/2018/feb/01/security-releases/"
        },
        {"url": "http://www.securitytracker.com/id/1040422"},
    ],
    "published_at": "2018-10-03T21:13:54Z",
    "updated_at": "2022-04-26T18:35:37Z",
    "withdrawn_at": None,
}


class SecurityAdvisoryTestCase(unittest.TestCase):
    def test_from_dict(self):
        advisory = SecurityAdvisory.from_dict(SECURITY_ADVISORY)

        self.assertEqual(advisory.cve_id, "CVE-2018-6188")
        self.assertEqual(advisory.ghsa_id, "GHSA-rf4j-j272-fj86")
        self.assertEqual(
            advisory.cvss.vector_string,
            "CVSS:3.0/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:N/A:N",
        )
        self.assertEqual(advisory.cvss.score, 7.5)
        self.assertRegex(advisory.summary, "^Django allows remote attacker.*")
        self.assertRegex(
            advisory.description,
            "^django\.contrib\.auth\.forms\.AuthenticationForm in Django 2\.0.*",
        )

        self.assertEqual(len(advisory.vulnerabilities), 2)
        vuln = advisory.vulnerabilities[0]
        self.assertEqual(vuln.package.ecosystem, "pip")
        self.assertEqual(vuln.package.name, "django")
        self.assertEqual(vuln.severity, Severity.HIGH)
        self.assertEqual(vuln.vulnerable_version_range, ">= 2.0.0, < 2.0.2")
        self.assertEqual(vuln.first_patched_version.identifier, "2.0.2")

        vuln = advisory.vulnerabilities[1]
        self.assertEqual(vuln.package.ecosystem, "pip")
        self.assertEqual(vuln.package.name, "django")
        self.assertEqual(vuln.severity, Severity.HIGH)
        self.assertEqual(vuln.vulnerable_version_range, ">= 1.11.8, < 1.11.10")
        self.assertEqual(vuln.first_patched_version.identifier, "1.11.10")

        self.assertEqual(len(advisory.cwes), 1)
        cwe = advisory.cwes[0]
        self.assertEqual(cwe.cwe_id, "CWE-200")
        self.assertEqual(
            cwe.name,
            "Exposure of Sensitive Information to an Unauthorized Actor",
        )

        self.assertEqual(len(advisory.identifiers), 2)
        identifier = advisory.identifiers[0]
        self.assertEqual(identifier.type, IdentifierType.GHSA)
        self.assertEqual(identifier.value, "GHSA-rf4j-j272-fj86")

        identifier = advisory.identifiers[1]
        self.assertEqual(identifier.type, IdentifierType.CVE)
        self.assertEqual(identifier.value, "CVE-2018-6188")

        self.assertEqual(len(advisory.references), 5)
        self.assertEqual(
            advisory.references[0].url,
            "https://nvd.nist.gov/vuln/detail/CVE-2018-6188",
        )

        self.assertEqual(
            advisory.published_at,
            datetime(2018, 10, 3, 21, 13, 54, tzinfo=timezone.utc),
        )
        self.assertEqual(
            advisory.updated_at,
            datetime(2022, 4, 26, 18, 35, 37, tzinfo=timezone.utc),
        )

        self.assertIsNone(advisory.withdrawn_at)


DEPENDABOT_ALERT = {
    "number": 2,
    "state": "dismissed",
    "dependency": {
        "package": {"ecosystem": "pip", "name": "django"},
        "manifest_path": "path/to/requirements.txt",
        "scope": "runtime",
    },
    "security_advisory": {
        "ghsa_id": "GHSA-rf4j-j272-fj86",
        "cve_id": "CVE-2018-6188",
        "summary": "Django allows remote attackers to obtain potentially sensitive information by leveraging data exposure from the confirm_login_allowed() method, as demonstrated by discovering whether a user account is inactive",
        "description": "django.contrib.auth.forms.AuthenticationForm in Django 2.0 before 2.0.2, and 1.11.8 and 1.11.9, allows remote attackers to obtain potentially sensitive information by leveraging data exposure from the confirm_login_allowed() method, as demonstrated by discovering whether a user account is inactive.",
        "vulnerabilities": [
            {
                "package": {"ecosystem": "pip", "name": "django"},
                "severity": "high",
                "vulnerable_version_range": ">= 2.0.0, < 2.0.2",
                "first_patched_version": {"identifier": "2.0.2"},
            },
            {
                "package": {"ecosystem": "pip", "name": "django"},
                "severity": "high",
                "vulnerable_version_range": ">= 1.11.8, < 1.11.10",
                "first_patched_version": {"identifier": "1.11.10"},
            },
        ],
        "severity": "high",
        "cvss": {
            "vector_string": "CVSS:3.0/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:N/A:N",
            "score": 7.5,
        },
        "cwes": [
            {
                "cwe_id": "CWE-200",
                "name": "Exposure of Sensitive Information to an Unauthorized Actor",
            }
        ],
        "identifiers": [
            {"type": "GHSA", "value": "GHSA-rf4j-j272-fj86"},
            {"type": "CVE", "value": "CVE-2018-6188"},
        ],
        "references": [
            {"url": "https://nvd.nist.gov/vuln/detail/CVE-2018-6188"},
            {"url": "https://github.com/advisories/GHSA-rf4j-j272-fj86"},
            {"url": "https://usn.ubuntu.com/3559-1/"},
            {
                "url": "https://www.djangoproject.com/weblog/2018/feb/01/security-releases/"
            },
            {"url": "http://www.securitytracker.com/id/1040422"},
        ],
        "published_at": "2018-10-03T21:13:54Z",
        "updated_at": "2022-04-26T18:35:37Z",
        "withdrawn_at": None,
    },
    "security_vulnerability": {
        "package": {"ecosystem": "pip", "name": "django"},
        "severity": "high",
        "vulnerable_version_range": ">= 2.0.0, < 2.0.2",
        "first_patched_version": {"identifier": "2.0.2"},
    },
    "url": "https://api.github.com/repos/octo-org/octo-repo/dependabot/alerts/2",
    "html_url": "https://github.com/octo-org/octo-repo/security/dependabot/2",
    "created_at": "2022-06-15T07:43:03Z",
    "updated_at": "2022-08-23T14:29:47Z",
    "dismissed_at": "2022-08-23T14:29:47Z",
    "dismissed_by": {
        "login": "octocat",
        "id": 1,
        "node_id": "MDQ6VXNlcjE=",
        "avatar_url": "https://github.com/images/error/octocat_happy.gif",
        "gravatar_id": "",
        "url": "https://api.github.com/users/octocat",
        "html_url": "https://github.com/octocat",
        "followers_url": "https://api.github.com/users/octocat/followers",
        "following_url": "https://api.github.com/users/octocat/following{/other_user}",
        "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
        "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
        "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
        "organizations_url": "https://api.github.com/users/octocat/orgs",
        "repos_url": "https://api.github.com/users/octocat/repos",
        "events_url": "https://api.github.com/users/octocat/events{/privacy}",
        "received_events_url": "https://api.github.com/users/octocat/received_events",
        "type": "User",
        "site_admin": False,
    },
    "dismissed_reason": "tolerable_risk",
    "dismissed_comment": "This alert is accurate but we use a sanitizer.",
    "fixed_at": None,
    "repository": {
        "id": 217723378,
        "node_id": "MDEwOlJlcG9zaXRvcnkyMTc3MjMzNzg=",
        "name": "octo-repo",
        "full_name": "octo-org/octo-repo",
        "owner": {
            "login": "octo-org",
            "id": 6811672,
            "node_id": "MDEyOk9yZ2FuaXphdGlvbjY4MTE2NzI=",
            "avatar_url": "https://avatars3.githubusercontent.com/u/6811672?v=4",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octo-org",
            "html_url": "https://github.com/octo-org",
            "followers_url": "https://api.github.com/users/octo-org/followers",
            "following_url": "https://api.github.com/users/octo-org/following{/other_user}",
            "gists_url": "https://api.github.com/users/octo-org/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octo-org/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octo-org/subscriptions",
            "organizations_url": "https://api.github.com/users/octo-org/orgs",
            "repos_url": "https://api.github.com/users/octo-org/repos",
            "events_url": "https://api.github.com/users/octo-org/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octo-org/received_events",
            "type": "Organization",
            "site_admin": False,
        },
        "private": True,
        "html_url": "https://github.com/octo-org/octo-repo",
        "description": None,
        "fork": False,
        "url": "https://api.github.com/repos/octo-org/octo-repo",
        "archive_url": "https://api.github.com/repos/octo-org/octo-repo/{archive_format}{/ref}",
        "assignees_url": "https://api.github.com/repos/octo-org/octo-repo/assignees{/user}",
        "blobs_url": "https://api.github.com/repos/octo-org/octo-repo/git/blobs{/sha}",
        "branches_url": "https://api.github.com/repos/octo-org/octo-repo/branches{/branch}",
        "collaborators_url": "https://api.github.com/repos/octo-org/octo-repo/collaborators{/collaborator}",
        "comments_url": "https://api.github.com/repos/octo-org/octo-repo/comments{/number}",
        "commits_url": "https://api.github.com/repos/octo-org/octo-repo/commits{/sha}",
        "compare_url": "https://api.github.com/repos/octo-org/octo-repo/compare/{base}...{head}",
        "contents_url": "https://api.github.com/repos/octo-org/octo-repo/contents/{+path}",
        "contributors_url": "https://api.github.com/repos/octo-org/octo-repo/contributors",
        "deployments_url": "https://api.github.com/repos/octo-org/octo-repo/deployments",
        "downloads_url": "https://api.github.com/repos/octo-org/octo-repo/downloads",
        "events_url": "https://api.github.com/repos/octo-org/octo-repo/events",
        "forks_url": "https://api.github.com/repos/octo-org/octo-repo/forks",
        "git_commits_url": "https://api.github.com/repos/octo-org/octo-repo/git/commits{/sha}",
        "git_refs_url": "https://api.github.com/repos/octo-org/octo-repo/git/refs{/sha}",
        "git_tags_url": "https://api.github.com/repos/octo-org/octo-repo/git/tags{/sha}",
        "hooks_url": "https://api.github.com/repos/octo-org/octo-repo/hooks",
        "issue_comment_url": "https://api.github.com/repos/octo-org/octo-repo/issues/comments{/number}",
        "issue_events_url": "https://api.github.com/repos/octo-org/octo-repo/issues/events{/number}",
        "issues_url": "https://api.github.com/repos/octo-org/octo-repo/issues{/number}",
        "keys_url": "https://api.github.com/repos/octo-org/octo-repo/keys{/key_id}",
        "labels_url": "https://api.github.com/repos/octo-org/octo-repo/labels{/name}",
        "languages_url": "https://api.github.com/repos/octo-org/octo-repo/languages",
        "merges_url": "https://api.github.com/repos/octo-org/octo-repo/merges",
        "milestones_url": "https://api.github.com/repos/octo-org/octo-repo/milestones{/number}",
        "notifications_url": "https://api.github.com/repos/octo-org/octo-repo/notifications{?since,all,participating}",
        "pulls_url": "https://api.github.com/repos/octo-org/octo-repo/pulls{/number}",
        "releases_url": "https://api.github.com/repos/octo-org/octo-repo/releases{/id}",
        "stargazers_url": "https://api.github.com/repos/octo-org/octo-repo/stargazers",
        "statuses_url": "https://api.github.com/repos/octo-org/octo-repo/statuses/{sha}",
        "subscribers_url": "https://api.github.com/repos/octo-org/octo-repo/subscribers",
        "subscription_url": "https://api.github.com/repos/octo-org/octo-repo/subscription",
        "tags_url": "https://api.github.com/repos/octo-org/octo-repo/tags",
        "teams_url": "https://api.github.com/repos/octo-org/octo-repo/teams",
        "trees_url": "https://api.github.com/repos/octo-org/octo-repo/git/trees{/sha}",
    },
}


class DependabotAlertTestCase(unittest.TestCase):
    def test_from_dict(self):
        alert = DependabotAlert.from_dict(DEPENDABOT_ALERT)

        self.assertEqual(alert.number, 2)
        self.assertEqual(alert.state, AlertState.DISMISSED)

        self.assertEqual(alert.dependency.package.ecosystem, "pip")
        self.assertEqual(alert.dependency.package.name, "django")
        self.assertEqual(
            alert.dependency.manifest_path, "path/to/requirements.txt"
        )
        self.assertEqual(alert.dependency.scope, DependencyScope.RUNTIME)

        self.assertEqual(alert.security_advisory.cve_id, "CVE-2018-6188")
        self.assertEqual(alert.security_advisory.ghsa_id, "GHSA-rf4j-j272-fj86")
        self.assertEqual(
            alert.security_advisory.cvss.vector_string,
            "CVSS:3.0/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:N/A:N",
        )
        self.assertEqual(alert.security_advisory.cvss.score, 7.5)
        self.assertRegex(
            alert.security_advisory.summary, "^Django allows remote attacker.*"
        )
        self.assertRegex(
            alert.security_advisory.description,
            "^django\.contrib\.auth\.forms\.AuthenticationForm in Django 2\.0.*",
        )

        self.assertEqual(len(alert.security_advisory.vulnerabilities), 2)
        vuln = alert.security_advisory.vulnerabilities[0]
        self.assertEqual(vuln.package.ecosystem, "pip")
        self.assertEqual(vuln.package.name, "django")
        self.assertEqual(vuln.severity, Severity.HIGH)
        self.assertEqual(vuln.vulnerable_version_range, ">= 2.0.0, < 2.0.2")
        self.assertEqual(vuln.first_patched_version.identifier, "2.0.2")

        vuln = alert.security_advisory.vulnerabilities[1]
        self.assertEqual(vuln.package.ecosystem, "pip")
        self.assertEqual(vuln.package.name, "django")
        self.assertEqual(vuln.severity, Severity.HIGH)
        self.assertEqual(vuln.vulnerable_version_range, ">= 1.11.8, < 1.11.10")
        self.assertEqual(vuln.first_patched_version.identifier, "1.11.10")

        self.assertEqual(len(alert.security_advisory.cwes), 1)
        cwe = alert.security_advisory.cwes[0]
        self.assertEqual(cwe.cwe_id, "CWE-200")
        self.assertEqual(
            cwe.name,
            "Exposure of Sensitive Information to an Unauthorized Actor",
        )

        self.assertEqual(len(alert.security_advisory.identifiers), 2)
        identifier = alert.security_advisory.identifiers[0]
        self.assertEqual(identifier.type, IdentifierType.GHSA)
        self.assertEqual(identifier.value, "GHSA-rf4j-j272-fj86")

        identifier = alert.security_advisory.identifiers[1]
        self.assertEqual(identifier.type, IdentifierType.CVE)
        self.assertEqual(identifier.value, "CVE-2018-6188")

        self.assertEqual(len(alert.security_advisory.references), 5)
        self.assertEqual(
            alert.security_advisory.references[0].url,
            "https://nvd.nist.gov/vuln/detail/CVE-2018-6188",
        )

        self.assertEqual(
            alert.security_advisory.published_at,
            datetime(2018, 10, 3, 21, 13, 54, tzinfo=timezone.utc),
        )
        self.assertEqual(
            alert.security_advisory.updated_at,
            datetime(2022, 4, 26, 18, 35, 37, tzinfo=timezone.utc),
        )

        self.assertIsNone(alert.security_advisory.withdrawn_at)

        self.assertEqual(alert.security_vulnerability.package.ecosystem, "pip")
        self.assertEqual(alert.security_vulnerability.package.name, "django")
        self.assertEqual(
            alert.security_vulnerability.vulnerable_version_range,
            ">= 2.0.0, < 2.0.2",
        )
        self.assertEqual(
            alert.security_vulnerability.first_patched_version.identifier,
            "2.0.2",
        )
        self.assertEqual(alert.security_vulnerability.severity, Severity.HIGH)

        self.assertEqual(
            alert.url,
            "https://api.github.com/repos/octo-org/octo-repo/dependabot/alerts/2",
        )
        self.assertEqual(
            alert.html_url,
            "https://github.com/octo-org/octo-repo/security/dependabot/2",
        )

        self.assertEqual(
            alert.created_at,
            datetime(2022, 6, 15, 7, 43, 3, tzinfo=timezone.utc),
        )
        self.assertEqual(
            alert.updated_at,
            datetime(2022, 8, 23, 14, 29, 47, tzinfo=timezone.utc),
        )
        self.assertEqual(
            alert.dismissed_at,
            datetime(2022, 8, 23, 14, 29, 47, tzinfo=timezone.utc),
        )

        self.assertEqual(alert.dismissed_by.login, "octocat")
        self.assertEqual(alert.dismissed_reason, DismissedReason.TOLERABLE_RISK)
        self.assertEqual(
            alert.dismissed_comment,
            "This alert is accurate but we use a sanitizer.",
        )

        self.assertIsNone(alert.fixed_at)
        self.assertIsNone(alert.auto_dismissed_at)

        self.assertEqual(alert.repository.id, 217723378)



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from unittest.mock import patch

from pontos.github.api.helper import DEFAULT_TIMEOUT
from pontos.github.script._parser import create_parser


class CreateParserTestCase(unittest.TestCase):
    def test_create_parser(self):
        parser = create_parser()
        args = parser.parse_args(
            ["--token", "123", "--timeout", "456", "my_script"]
        )

        self.assertEqual(args.token, "123")
        self.assertEqual(args.timeout, 456)
        self.assertEqual(args.script, "my_script")

    @patch.dict("os.environ", {"GITHUB_TOKEN": "987"})
    def test_parse_token(self):
        parser = create_parser()
        args = parser.parse_args(["my_script"])
        self.assertEqual(args.token, "987")
        self.assertEqual(args.script, "my_script")

        args = parser.parse_args(["--token", "123", "my_script"])

        self.assertEqual(args.token, "123")
        self.assertEqual(args.script, "my_script")

    def test_parse_timeout(self):
        parser = create_parser()
        args = parser.parse_args(["my_script"])
        self.assertEqual(args.script, "my_script")
        self.assertEqual(args.timeout, DEFAULT_TIMEOUT)

        args = parser.parse_args(["--timeout", "666", "my_script"])

        self.assertEqual(args.timeout, 666)
        self.assertEqual(args.script, "my_script")



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from ._cpe import ANY, CPE, NA, CPEParsingError, Part

"""
Module for parsing and handling Common Platform Enumeration (CPE) information
"""

__all__ = (
    "ANY",
    "NA",
    "CPEParsingError",
    "Part",
    "CPE",
)



# SPDX-FileCopyrightText: 2021-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .create import CreateReleaseCommand, CreateReleaseReturnValue
from .helper import ReleaseType, find_signing_key, get_git_repository_name
from .main import main
from .sign import SignatureError, SignCommand, SignReturnValue

__all__ = (
    "ReleaseType",
    "get_git_repository_name",
    "find_signing_key",
    "CreateReleaseCommand",
    "CreateReleaseReturnValue",
    "SignCommand",
    "SignatureError",
    "SignReturnValue",
    "main",
)
